#!/usr/bin/env python
# Parallel data scraped from ELRC-SHARE using https://github.com/kpu/elrc-scrape
# Author: Kenneth Heafield [mtdata (at) kheafield (dot) com] 

from pathlib import Path
from mtdata.index import Index, Entry, DatasetId
REFS_FILE = Path(__file__).parent / 'elrc_share.tsv'


def load_all(index: Index):
    with open(REFS_FILE, encoding='utf-8') as data:
        for line in data:
            l1, l2, num, short, name, info, download, licenses, in_paths = line.split('\t', maxsplit=8)
            dataset_name = short.lower().replace(':', '_').replace('__', '_').replace('__', '_')
            in_paths = in_paths.strip().split('\t')
            ent = Entry(did=DatasetId(group='ELRC', name=dataset_name, version='1', langs=(l1, l2)),
                    url=download, filename="ELRC_" + str(num) + ".zip", in_ext='tmx', in_paths=in_paths)
            index.add_entry(ent)



#!/usr/bin/env python
#
# Author: Thamme Gowda [tg (at) isi (dot) edu]
# Created: 4/5/20
import hashlib
from pathlib import Path
from mtdata import log, pbar_man, cache_dir as CACHE_DIR, MTDataException
from mtdata.cache import Cache
from mtdata.index import INDEX, Entry, DatasetId, LangPair
from mtdata.iso.bcp47 import bcp47, BCP47Tag
from mtdata.parser import Parser
from mtdata.utils import IO
from typing import Optional, List, Tuple
from itertools import zip_longest
import collections as coll
import json

DEF_COMPRESS = 'gz'


class Dataset:

    def __init__(self, dir: Path, langs: LangPair, cache_dir: Path, drop_train_noise=True,
                 drop_test_noise=False, drop_dupes=False, drop_tests=False, compress=False):
        self.dir = dir
        self.langs = langs
        assert len(langs) == 2, 'Only parallel datasets are supported for now and expect two langs'
        self.cache = Cache(cache_dir)

        self.train_parts_dir = dir / 'train-parts'  # will be merged later
        self.tests_dir = dir / 'tests'  # wont be merged
        self.train_parts_dir.mkdir(parents=True, exist_ok=True)
        self.tests_dir.mkdir(parents=True, exist_ok=True)
        self.drop_train_noise = drop_train_noise
        self.drop_test_noise = drop_test_noise
        self.drop_dupes = drop_dupes  # in training only
        self.drop_tests = drop_tests  # in training only

    @classmethod
    def resolve_entries(cls, dids: List[DatasetId]):
        inp_dids = set(dids)
        assert len(inp_dids) == len(dids), f'{dids} are not unique.'
        entries = []
        for did in inp_dids:
            if did in INDEX:
                entries.append(INDEX[did])
            else:
                raise Exception(f'Could not find {did}; try "mtdata list | grep -i {did}" to locate it')
        return entries

    @classmethod
    def prepare(cls, langs, out_dir: Path, train_dids: Optional[List[DatasetId]] = None,
                test_dids: Optional[List[DatasetId]] = None, dev_did: Optional[DatasetId] = None,
                cache_dir: Path = CACHE_DIR, merge_train=False, drop_noise: Tuple[bool, bool] = (True, False),
                compress=False, drop_dupes=False, drop_tests=False):
        drop_train_noise, drop_test_noise = drop_noise
        assert langs, 'langs required'
        assert train_dids or test_dids, 'Either train_names or test_names should be given'
        # First, resolve and check if they exist before going to process them.
        # Fail early for typos in name
        train_entries, test_entries = None, None
        if test_dids:
            test_entries = cls.resolve_entries(test_dids)
        if train_dids:
            train_entries = cls.resolve_entries(train_dids)

        dataset = cls(dir=out_dir, langs=langs, cache_dir=cache_dir,
                      drop_train_noise=drop_train_noise, drop_test_noise=drop_test_noise,
                      drop_dupes=drop_dupes, drop_tests=drop_tests)
        if test_entries:  # tests are smaller so quicker; no merging needed
            dataset.add_test_entries(test_entries)
        if dev_did:
            dev_entry = cls.resolve_entries([dev_did])[0]
            dataset.add_dev_entry(dev_entry)

        if train_entries:  # this might take some time
            dataset.add_train_entries(train_entries, merge_train=merge_train, compress=compress)
        return dataset

    def hash_all_held_outs(self):
        lang1, lang2 = self.langs
        paired_files = self.find_bitext_pairs(self.tests_dir, lang1, lang2)
        paired_hashes = set()
        seg_hashes = set()
        for name, (if1, if2) in paired_files.items():
            for seg1, seg2 in self.read_parallel(if1, if2):
                paired_hashes.add(hash((seg1, seg2)))
                paired_hashes.add(hash((seg2, seg1)))
                seg_hashes.add(hash(seg1))
                seg_hashes.add(hash(seg2))
        return paired_hashes, seg_hashes

    def add_train_entries(self, entries, merge_train=False, compress=False):
        self.add_parts(self.train_parts_dir, entries, drop_noise=self.drop_train_noise,
                       compress=compress, desc='Training sets')
        if not merge_train:
            return
        lang1, lang2 = self.langs
        paired_files = self.find_bitext_pairs(self.train_parts_dir, lang1, lang2)

        log.info(f"Going to merge {len(paired_files)} files as one train file")

        compress_ext = f'.{DEF_COMPRESS}' if compress else ''
        l1_ext = f'{lang1}{compress_ext}'
        l2_ext = f'{lang2}{compress_ext}'
        of1 = self.dir / f'train.{l1_ext}'
        of2 = self.dir / f'train.{l2_ext}'
        of3 = self.dir / f'train.meta.{DEF_COMPRESS}'

        counts = dict(total=coll.defaultdict(int),
                      dupes_skips=coll.defaultdict(int),
                      test_overlap_skips=coll.defaultdict(int),
                      selected=coll.defaultdict(int))
        train_hashes = set()
        tests_pair_hashes, tests_seg_hashes = set(), set()
        if self.drop_tests:
            tests_pair_hashes, tests_seg_hashes = self.hash_all_held_outs()

        with IO.writer(of1) as w1, IO.writer(of2) as w2, IO.writer(of3) as w3:
            with pbar_man.counter(color='green', total=len(paired_files), unit='it', desc="Merging",
                                  autorefresh=True) as pbar:
                for name, (if1, if2) in paired_files.items():
                    for seg1, seg2 in self.read_parallel(if1, if2):
                        counts['total'][name] += 1
                        if self.drop_dupes or self.drop_tests:
                            hash_val = hash((seg1, seg2))
                            if self.drop_tests and (hash_val in tests_pair_hashes
                                                    or hash(seg1) in tests_seg_hashes
                                                    or hash(seg2) in tests_seg_hashes):
                                counts['test_overlap_skips'][name] += 1
                                continue
                            if self.drop_dupes:
                                if hash_val in train_hashes:
                                    counts['dupes_skips'][name] += 1
                                    continue
                                train_hashes.add(hash_val)
                        w1.write(seg1 + '\n')
                        w2.write(seg2 + '\n')
                        w3.write(name + '\n')
                        counts['selected'][name] += 1
                    pbar.update()

        stats = dict(selected=sum(counts['selected'].values()),
                     total=sum(counts['total'].values()),
                     counts=counts)

        stats_msg = json.dumps(stats, indent=2)
        log.info('Train stats:\n' + stats_msg)
        IO.write_lines(self.dir / 'train.stats.json', stats_msg)
        return counts

    @classmethod
    def find_bitext_pairs(cls, dir_path: Path, lang1: BCP47Tag, lang2: BCP47Tag):
        if lang1.is_compatible(lang2):
            raise Exception(f"Unable to merge for {lang1}-{lang2}; it can result in unpredictable behavior.")
        paired_files = {}
        for path in dir_path.glob("*.*"):
            if path.name.startswith("."):
                continue
            parts = path.name.split(".")
            assert len(parts) >= 2, f'Invalid file name {path.name}; Unable to merge parts'
            if parts[-1] == DEF_COMPRESS:
                parts = parts[:-1]
            *did, ext = parts  # did can have a dot e.g. version 7.1
            did = '.'.join(did)
            # dids, ext = parts[:2]
            ext = bcp47(ext)
            if did not in paired_files:
                paired_files[did] = [None, None]
            if lang1.is_compatible(ext):
                assert not lang2.is_compatible(ext)
                paired_files[did][0] = path
            elif lang2.is_compatible(ext):
                paired_files[did][1] = path
            else:
                raise Exception(f"Unable to decide the side of train-part {path}; ext={ext}: we have {lang1}-{lang2}")
        for did, (f1, f2) in paired_files.items():
            assert f1 and f1.exists(), f'Invalid state: part {did} does not have pair, or pair is are removed'
        return paired_files

    @classmethod
    def read_parallel(cls, file1: Path, file2: Path):
        with IO.reader(file1) as r1, IO.reader(file2) as r2:
            for seg1, seg2 in zip_longest(r1, r2):
                if seg1 is None or seg2 is None:
                    raise Exception(f'{file1} {file2} have unequal num of lines. This is an error.')
                yield seg1.strip(), seg2.strip()

    def add_test_entries(self, entries):
        self.add_parts(self.tests_dir, entries, drop_noise=self.drop_test_noise, desc='Held-out sets')
        if len(entries) <= 4:
            for i, entry in enumerate(entries, start=1):
                self.link_to_part(entry, self.tests_dir, f"test{i}")

    def link_to_part(self, entry, data_dir, link_name):
        """Create link such as test, dev"""
        l1_path, l2_path = self.get_paths(data_dir, entry)
        l1_path, l2_path = l1_path.relative_to(self.dir), l2_path.relative_to(self.dir)
        l1_link = self.dir / f'{link_name}.{self.langs[0]}'
        l2_link = self.dir / f'{link_name}.{self.langs[1]}'
        for lnk in [l1_link, l2_link]:
            lnk.unlink(missing_ok=True)
        if BCP47Tag.are_compatible(self.langs[0], entry.did.langs[0]):
            assert not BCP47Tag.are_compatible(self.langs[0], entry.did.langs[1])
            # cool! no swapping needed
        elif BCP47Tag.are_compatible(self.langs[0], entry.did.langs[1]):
            l1_path, l2_path = l2_path, l1_path  # swapped
        else:
            raise Exception("This should not be happening! :(")

        l1_link.symlink_to(l1_path)
        l2_link.symlink_to(l2_path)

    def add_dev_entry(self, entry):
        n_good, n_bad = self.add_part(self.tests_dir, entry, drop_noise=self.drop_test_noise)
        log.info(f"{entry.did} : found {n_good:} segments and {n_bad:} errors")
        # create a link
        self.link_to_part(entry, self.tests_dir, "dev")

    def add_parts(self, dir_path, entries, drop_noise=False, compress=False, desc=None, fail_on_error=False):
        with pbar_man.counter(color='blue', leave=False, total=len(entries), unit='it', desc=desc,
                              autorefresh=True) as pbar:
            for ent in entries:
                try:
                    n_good, n_bad = self.add_part(dir_path=dir_path, entry=ent, drop_noise=drop_noise,
                                                  compress=compress)
                    log.info(f"{ent.did.name} : found {n_good:} segments and {n_bad:} errors")
                    pbar.update(force=True)
                except MTDataException as e:
                    log.error(f"Unable to add {ent.did}: {e}")
                    if fail_on_error:
                        raise e
                    else:
                        log.warning(e)

    @classmethod
    def get_paths(cls, dir_path: Path, entry: Entry, compress=False) -> Tuple[Path, Path]:
        l1_ext = str(entry.did.langs[0])
        l2_ext = str(entry.did.langs[1])
        if compress:
            l1_ext += f'.{DEF_COMPRESS}'
            l2_ext += f'.{DEF_COMPRESS}'
        l1 = dir_path / f'{entry.did}.{l1_ext}'
        l2 = dir_path / f'{entry.did}.{l2_ext}'
        return l1, l2

    def add_part(self, dir_path: Path, entry: Entry, drop_noise=False, compress=False):
        path = self.cache.get_entry(entry)
        # swap = entry.is_swap(self.langs)
        parser = Parser(path, ext=entry.in_ext or None, ent=entry)
        # langs = '_'.join(str(lang) for lang in self.langs)
        # Check that files are written in correct order
        l1, l2 = self.get_paths(dir_path, entry, compress=compress)
        io_args = dict(encoding='utf-8', errors='ignore')
        with IO.writer(l1, **io_args) as f1, IO.writer(l2, **io_args) as f2:
            count, skips, noise = 0, 0, 0
            for rec in parser.read_segs():
                rec = rec[:2]  # get the first two recs
                if len(rec) != 2:
                    skips += 1
                    continue
                if drop_noise and entry.is_noisy(seg1=rec[0], seg2=rec[1]):
                    skips += 1
                    noise += 1
                    continue
                sent1, sent2 = [s.strip() for s in rec]
                if not sent1 or not sent2:
                    skips += 1
                    continue
                # if swap:
                #    sent2, sent1 = sent1, sent2
                sent1 = sent1.replace('\n', ' ').replace('\t', ' ').replace('\r', ' ')
                sent2 = sent2.replace('\n', ' ').replace('\t', ' ').replace('\r', ' ')
                f1.write(f'{sent1}\n')
                f2.write(f'{sent2}\n')
                count += 1
            msg = f'Looks like an error. {count} segs are valid {skips} are invalid: {entry}'
            assert count > 0, msg
            if skips > count:
                log.warning(msg)
            if noise > 0:
                log.info(f"{entry}: Noise : {noise:,}/{count:,} => {100 * noise / count:.4f}%")
            log.info(f"wrote {count} lines to {l1} == {l2}")
        return count, skips


def rreplace(text: str, cut: str, place: str):
    """ right replace: https://stackoverflow.com/a/9943875/1506477
        >>> 'XXX'.join('mississippi'.rsplit('iss', 1))
        'missXXXippi'
    """
    return place.join(text.split(cut, 1))



#!/usr/bin/env python
#
#
# Author: Thamme Gowda
# Created: 10/12/21
from mtdata.iso.iso639_3 import CODES as iso3_codes, name_to_code
from mtdata.iso.iso639_2 import CODE2_TO_3, code2_to_code3_name
from mtdata.iso.iso639_1 import ISO693_1_to_3 as code1_to_3
from mtdata.iso.custom import CUSTOM_TO_3 as custom_to_3


def iso3_code(lang: str, fail_error=False, default=None) -> str:
    """
    Lookup 693_3 code of a language
    :param lang: language name or a code, maybe of 2 or 3 letters
    :param default: return this value when no iso code is found
    :param fail_error: raise error when no ISO code is found
    :return: ISO 639_3 code
    """
    lang = lang.lower()
    part1 = lang.split('-')[0]     # BCP47
    lookups = (lang, part1) if lang != part1 else (lang,)
    for lang in lookups:
        if lang in iso3_codes:
            return lang
        if lang in CODE2_TO_3:
            _, name = code2_to_code3_name(lang)
            iso3_code = name_to_code(name)
            if iso3_code:
                return iso3_code
        if lang in code1_to_3:
            return code1_to_3[lang]
        if name_to_code(lang, None):
            return name_to_code(lang)
        if lang in custom_to_3:  # at last
            return custom_to_3[lang]

    if fail_error:
        raise Exception(f"Unable to find ISO 639-3 code for '{lang}'. "
                        f"Please run\npython -m mtdata.iso | grep -i <name>\n"
                        f"to know the 3 letter ISO code for the language.")
    return default



#!/usr/bin/env python
#
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 4/4/20

__version__ = '0.3.0'
__description__ = 'mtdata is a tool to download datasets for machine translation'
__author__ = 'Thamme Gowda'

import logging as log
from pathlib import Path
import os


debug_mode = False
_log_format = '%(asctime)s %(module)s.%(funcName)s:%(lineno)s %(levelname)s:: %(message)s'
log.basicConfig(level=log.INFO, datefmt='%Y-%m-%d %H:%M:%S', format=_log_format)
cache_dir = Path(os.environ.get('MTDATA', '~/.mtdata')).expanduser()
cached_index_file = cache_dir / f'mtdata.index.{__version__}.pkl'

try:
    import enlighten
    pbar_man = enlighten.get_manager()
except ImportError as e:
    log.warning("enlighten package maybe required. please run 'pip install englighten'")
    log.warning(e)


class MTDataException(Exception):
    pass



from mtdata.index import Index, Entry, DatasetId

wiki_titles = """04j515tc7j2jgpj,arbg
fzrq1bg6rzx56iq,arcs
faka5afgeucyft7,arda
nc8gth1fv4zhsof,arde
73tosz4ire4um0n,arel
s08r9yw1hxcbecj,aren
jc6yai66mn6bh5l,ares
mdigquqrasof07s,arfa
7uecxuwkr5iwvgc,arfi
ib89l8dckxbq1xs,arfr
jaxmqqcmp75b2sy,arhu
drsjnpopadedd1p,arit
jayauqsn0ww8gr2,arja
a49vjagxu7bilwk,arko
gaxeih66haljo27,arnl
1k7ihxd8fq0fcda,arpl
aj4ofxjatqf11ho,arpt
81yx0jeximdh330,arro
qv7bij8dwgkr1yd,arru
aixl0t4w3nj1on8,arsv
bg09c7d4rqghae9,artr
02mpbq5rwz65x3y,arzh
uxfpkgzqc5ah3lm,bgcs
7p3aeedjatb39r8,bgda
582v4xlc878o88c,bgde
4q18h8l6gaz8ktx,bgel
cxovv7t0zhvvkw2,bgen
15pdxb256xz840o,bges
a953mwj173y878u,bgfa
pqatqz66cotz799,bgfi
tmcq40bgnmce3t2,bgfr
yvkdg9mu6ckaxc0,bghu
kck6ixrx2v9wtim,bgit
s5v37odvb4dufx6,bgja
ebgujhdjm5vkxpw,bgko
gjzywe3pmso0dkz,bgnl
lsrvwqvd2v7hnhw,bgpl
vfc3aizbjtnbtio,bgpt
gwvtxncgj2fubbt,bgro
pxdl3hkv4iyluyt,bgru
3spvq90nnjo56mi,bgsv
cc4gr923fz2p1rt,bgtr
yml3m6ibxiqy2yf,bgzh
16zdu0hrgs6e8e7,csda
aorw1fqdirusgjy,csde
5fmsqmbnxzubvry,csel
rq29gx3f6jvm0uh,csen
3hfpgi6do85m9w9,cses
pvqlxrtw5uka3lo,csfa
iny694ip5gpjw7f,csfi
ft21z297hchzxk4,csfr
gb963u2wi29o3l1,cshu
wqldo99ral75npg,csit
63lodxfy8em5j5i,csja
zy7vg2a3eitzkzc,csko
pwj6rfs7mxbzf7e,csnl
vztt19ld7l3tc1t,cspl
kn1h8o52ucfu5fz,cspt
feosoxiqv3jx4it,csro
l1p3mnctkyowh7k,csru
sbzxrai9g2u9rk7,cssv
uexjhfm7r67f4uc,cstr
bmnwumqfu08phhe,cszh
cothkbkina12fta,dade
vr20cg5995lce0f,dael
6vdl57ii4lwfl7h,daen
h4cfk24r7iukjwa,daes
dec2i92flwo0j98,dafa
palsd7wxsnj7ntb,dafi
n6lhykj6ju21u6t,dafr
d7cljp7x4yhnfkz,dahu
fpkv82d9evkg6am,dait
n3hd4roiob4ic0c,daja
lkoj2p06gw93nsi,dako
nby1fqfz28lnp81,danl
5n3al8y17k6fl1g,dapl
t83jb8kupejcq6u,dapt
httb0wod5r558dt,daro
pglodclo3p12r1q,daru
w4cga8jt3zsbli1,dasv
ngdluw3il43swf4,datr
gjzhc96anwtpdmb,dazh
hgmeyvavoges6hc,deel
h1wqni2a8uapmmt,deen
b5bk2yquuunm702,dees
ye07dzb4vemzrbw,defa
0gpeyfabsx3qkqo,defi
0pw75p9bv1850fu,defr
yrydz53ol6fmdnj,dehu
yo20zsmqiap8wx7,deit
ccnys6sziztd5gz,deja
enpulumwk4pkogg,deko
6jj8z6aq4qizumb,denl
ju6pt60z00r1q18,depl
uypl3ez2o57c3a0,dept
xk6rw8nwohcbyl8,dero
h6ed7vnn3qny5iu,deru
vvpbwtcq9pns50e,desv
l9ave94viu8nfqx,detr
hcjto7we7vacufu,dezh
jbovwht1zjfmbnm,elen
odmauxqvlzy2cqv,eles
cewd7x04kkxke15,elfa
uzm3aa25e6eg53z,elfi
e4qg1a161cl587n,elfr
p0xajma22nr9idl,elhu
14e4czry96r47kt,elit
c03d5wwn1t7vnar,elja
24zzf9erxrwstlo,elko
y54cqp4oiq44jvq,elnl
qec37lk6cmr6n7h,elpl
ew1rhmr1r8bqxw8,elpt
xwitiqqw5wx8cpe,elro
lrgzd4ndp1ivojo,elru
dg2l31x6k7t3n7j,elsv
5yslv5wrc6076ri,eltr
03ox5ay1l2eq1nb,elzh
7kb21jpaegtdprs,enes
km8zipw2k7bb6bz,enfa
vswatv073ncvmbv,enfi
ic8gp6kwi64jkgw,enfr
5n7o2rqrmjr9lxe,enhu
htkvdpm83803w8g,enit
6qpcq4jgm4mjfo5,enja
vz7k6t3z06vm96m,enko
p1cvbteuzs4hhq9,ennl
fgt38mk22kgjzv5,enpl
7hxuj0kmd51ltgs,enpt
rm1zju1m7ycw176,enro
13zhw7fnoioh00q,enru
h3tepdu06qcg17j,ensv
n1fjhk9lpfql9ns,entr
9d0polb8rude9ha,enzh
hrhcc6avil1e8ck,esfa
c7dxm02j955deey,esfi
t2j3216m8it8eel,esfr
6jcx1vnsuk2d8fk,eshu
eudnt94d1borcbz,esit
boem8qd3hw75mhm,esja
st9jp8b4s3kpwm4,esko
h7x8ibovv5ntf2p,esnl
7bgtl8w1ycquvxj,espl
z6b0r3ak8knjvf1,espt
oeblr9pal4svdtu,esro
4ylw6i1omeft7z5,esru
e0h78kv29s66k14,essv
eafh77vy898ip3p,estr
n3aoersqta9a333,eszh
etr6f5717dmq1pj,fafi
oqh9o2rugeb089e,fafr
xfwh15uuzp0cegk,fahu
5n63tf7wylm3cxa,fait
kx9d806liuotmfq,faja
a3w6u0x0enpzyj0,fako
jkbjfk74l8caqg5,fanl
wpp3e0hd8tdsoyy,fapl
3ywvk747hxvra5r,fapt
ugtsfl4qgsv4113,faro
usd1hp2uu218k7l,faru
mp3rvp4iezgo6pj,fasv
taihf6u5qb6b83z,fatr
t3podec9xy4dwix,fazh
x8s0qaqibz6lltn,fifr
elqgtz8pesw1h8h,fihu
brcaedpruz1tces,fiit
s1e4e3y6hnqk2g4,fija
nkyo12831b2yslp,fiko
0r3lubi5d44qqr1,finl
z2d3wy7mvioepuf,fipl
7mnut4bhh1hsejy,fipt
0u2wko45er5c90z,firo
u7df3hd2hpp1yns,firu
73mgwa15eh2rjp3,fisv
ozif8nv89vqer81,fitr
0w9btvcj6kdnteh,fizh
9lqe6u1hz1zevr2,frhu
03sbabvthbqgdzz,frit
rv5uz7hkveqeql8,frja
olcp01loge9kmo9,frko
2pd43xmpacn4k2r,frnl
ckbwgjiu8191jyl,frpl
8on0fd3wd38jw4j,frpt
1imns16wcgv7595,frro
t47k1qm2yeyysiu,frru
3xtjph0tvay2ecz,frsv
inkoca7pf9bcpt6,frtr
hxyx23pncd7j2y2,frzh
yn30cv2swhomlue,huit
yix222t5lrx47yy,huja
cm8z4rn8kii78ji,huko
5mlxkl7rue9wtfs,hunl
5va5qcnlpptxquk,hupl
2lt9ffvp0bordpt,hupt
mlnxm1h83dxuxn0,huro
rl7qh7ll73nz1qo,huru
att8du4i79kvnj4,husv
6hvq1dr7pwm33dw,hutr
j6h63j2xgwndy1r,huzh
jkyf7ewjrlr8798,itja
vwmra6xuezw7w8f,itko
zl68e5u9orkyvch,itnl
mtzh6t0igk4ge5n,itpl
5n8wob9spf5db07,itpt
0zg9xed1qnsbu6u,itro
a9n293jqwi11gpr,itru
4s3e0tmo8cu7cen,itsv
l7nt03iokcxu11b,ittr
ycmny5cy1p02jkx,itzh
03lgeo6yugf5jht,jako
7foo73ru8ilmlwb,janl
u1rlz7gsih79pg0,japl
s50ygbigtxxbbqt,japt
iyl9mlknrunzcta,jaro
h16okcs9x8a4l5w,jaru
9agu7u5jjazx8d9,jasv
ukg1ek63mjx0g04,jatr
aqchhx3krswmqzn,jazh
j8fzv0ckxpxixvf,konl
wqg4yvbxe8dhw2x,kopl
teeefh0ss6h00cs,kopt
p5rkkhe320xh3f0,koro
k1kl5q3nzqhqiny,koru
ri1tsfzdkcytjx1,kosv
bg3ccbck5skhob0,kotr
gpve56kb43lnvy6,kozh
z4to6kujh8rwss5,nlpl
lqri0zguqy2424c,nlpt
zty96o9d9390usv,nlro
ck75o5t2zu3e6nr,nlru
meyy1k475gc8ykn,nlsv
riusgvccxbsiy0l,nltr
31epwbom2i6e3p0,nlzh
612ii879zckadtf,plpt
7qdtp2c5qnop0a8,plro
yq5cn5txawdvum5,plru
iqmq7ej3nyd71kj,plsv
e73hv4ejwktqtus,pltr
200xb0bkk1ti0x6,plzh
0wbt2u9rm4l4j5y,ptro
ab5dfdl07j06dw3,ptru
hkc20tgy5ifldof,ptsv
ljbxam0f2l7kkgu,pttr
0eegtffi5xemfnl,ptzh
63pt63fwgh219k8,roru
orhildoo8ozxap7,rosv
jc4c1uzh6ftq9o9,rotr
sow4dk9p00xxslg,rozh
huiyn8hjpq5lizv,rusv
8lx5tz7x1cyil3d,rutr
jcbphinsu1oedtb,ruzh
ldlxxbhembwmoec,svtr
f6lhyojcnua3od6,svzh
a4iffiidjb8he13,trzh"""


def load_all(index: Index):
    url_ptn = 'https://www.dropbox.com/s/{uid}/wikititles-2014_{l1}{l2}.tgz?dl=1'
    rows = [row.split(',') for row in wiki_titles.splitlines()]
    for row in rows:
        uid, pair = row
        assert len(pair) == 4
        l1, l2 = pair[:2], pair[2:]
        url = url_ptn.format(uid=uid, l1=l1, l2=l2)
        in_file = f'wikititles-2014_{l1}{l2}'
        ent = Entry(did=DatasetId(group='LinguaTools', name=f'wikititles', version='2014', langs=(l1, l2)),
                    url=url, ext='tgz', in_ext='txt', in_paths=[f'{in_file}.{l1}', f'{in_file}.{l2}'])
        index.add_entry(ent)


if __name__ == '__main__':
    index = Index()
    load_all(index)
    print("loaded", len(index))



#!/usr/bin/env python
#
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 5/8/20
import logging as log

log.basicConfig(level=log.INFO)


def main(langs=None):
    from mtdata.iso import iso3_code
    from mtdata.iso.iso639_3 import code_to_name, data as ISO639_3
    args = parse_args()
    langs = langs or args.get('langs', [])
    brief = args.get('brief')
    if not langs:
        print(ISO639_3)
    else:
        if not brief:
            print(f"Input\tISO639_3\tName")
        for lang in langs:
            iso_code = iso3_code(lang, fail_error=brief)
            iso_name = code_to_name(iso_code) if iso_code else '-none-'
            if brief:
                assert iso_code, f'Unable to resolve {lang} to valid language code.'
                print(f"{iso_code}\t{iso_name}")
            else:
                iso_code = iso_code or '-none-'
                print(f"{lang}\t{iso_code}\t{iso_name}")


def parse_args():
    import argparse
    p = argparse.ArgumentParser(prog='python -m mtdata.iso', description="ISO 639-3 lookup tool")
    p.add_argument("langs", nargs='*', help="Language code or name that needs to be looked up. "
                                            "When no language code is given, all languages are listed.")
    p.add_argument('-b', '--brief', action='store_true', help="be brief; do crash on error inputs")

    return vars(p.parse_args())


if __name__ == '__main__':
    main()



#!/usr/bin/env python
#
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 4/8/20
from mtdata.index import Entry, Index, DatasetId, bcp47
import itertools


def load(index: Index):
    group_id = 'Statmt'
    WMT13_CCRAWL = "http://www.statmt.org/wmt13/training-parallel-commoncrawl.tgz"
    WMT14_CITE = index.ref_db.get_bibtex('ws-2014-statistical')
    for l1 in ['de', 'cs', 'fr', 'ru', 'es']:
        l2 = 'en'
        f1 = f'commoncrawl.{l1}-en.{l1}'
        f2 = f'commoncrawl.{l1}-en.en'
        data_id = DatasetId(group=group_id, name='commoncrawl_wmt13', version='1', langs=(l1, l2))
        index.add_entry(Entry(did=data_id, url=WMT13_CCRAWL,
                              filename='wmt13_parallel_commoncrawl.tgz',
                              in_paths=[f1, f2], in_ext='txt', cite=WMT14_CITE))

    # === WMT 13 release of europarl_v7 ===
    for l1 in ['cs', 'de', 'fr', 'es']:
        l2 = 'en'
        f1 = f'training/europarl-v7.{l1}-{l2}.{l1}'
        f2 = f'training/europarl-v7.{l1}-{l2}.{l2}'
        data_id = DatasetId(group=group_id, name='europarl_wmt13', version='7', langs=(l1, l2))
        index.add_entry(Entry(did=data_id, url="http://www.statmt.org/wmt13/training-parallel-europarl-v7.tgz",
                              filename="wmt13_europarl_v7.tgz",
                              in_paths=[f1, f2], in_ext='txt', cite=WMT14_CITE))

    # ==== WMT 18  news commentary v13 ===
    for l1 in ['cs', 'de', 'ru', 'zh']:
        l2 = 'en'
        f1 = f'training-parallel-nc-v13/news-commentary-v13.{l1}-{l2}.{l1}'
        f2 = f'training-parallel-nc-v13/news-commentary-v13.{l1}-{l2}.{l2}'
        data_id = DatasetId(group=group_id, name='news_commentary_wmt18', version='13', langs=(l1, l2))
        index.add_entry(Entry(did=data_id,
                              url="http://data.statmt.org/wmt18/translation-task/training-parallel-nc-v13.tgz",
                              filename="wmt18_news_commentary_v13.tgz",
                              in_paths=[f1, f2], in_ext='txt', cite=WMT14_CITE))

    # === Europarl V9 corpus
    EUROPARL_v9 = 'http://www.statmt.org/europarl/v9/training/europarl-v9.%s-%s.tsv.gz'
    cite = index.ref_db.get_bibtex('koehn2005europarl')
    for pair in ['de en', 'cs en', 'cs pl', 'es pt', 'fi en', 'lt en']:
        l1, l2 = pair.split()
        index.add_entry(Entry(did=DatasetId(group=group_id, name='europarl', version='9', langs=(l1, l2)),
                              url=EUROPARL_v9 % (l1, l2), cite=cite))

    # === Europarl V7 corpus
    EUROPARL_v7 = 'http://www.statmt.org/europarl/v7/%s-%s.tgz'
    cite = index.ref_db.get_bibtex('bojar-etal-2017-findings')
    for l1 in 'bg cs da de el es et fi fr hu it lt lv nl pl pt ro sk sl sv'.split():
        l2 = 'en'
        src = f'europarl-v7.{l1}-{l2}.{l1}'
        ref = f'europarl-v7.{l1}-{l2}.{l2}'
        index.add_entry(Entry(
            did=DatasetId(group=group_id, name='europarl', version='7', langs=(l1, l2)), in_paths=[src, ref],
                              url=EUROPARL_v7 % (l1, l2), in_ext='txt', cite=cite))

    # === Digital Corpus of European Parliament
    index.add_entry(Entry(did=DatasetId(group=group_id, name='dcep_wmt17', version='1', langs=(l1, l2)),
                          in_paths=['*/*.lv', f'*/*.en'], cite=cite, in_ext='txt',
                          url='http://data.statmt.org/wmt17/translation-task/dcep.lv-en.v1.tgz'))
    index.add_entry(Entry(did=DatasetId(group=group_id, name='books_wmt17', version='1', langs=(l1, l2)),
                          in_paths=['*/*.lv', f'*/*.en'], cite=cite, in_ext='txt',
                          url='http://data.statmt.org/wmt17/translation-task/books.lv-en.v1.tgz'))

    # === News Commentary v14
    NEWSCOM_v14 = "http://data.statmt.org/news-commentary/v14/training/news-commentary-v14.%s-%s.tsv.gz"
    cite = index.ref_db.get_bibtex('bojar-etal-2018-findings')
    for pair in ['ar cs', 'ar de', 'ar en', 'ar es', 'ar fr', 'ar hi', 'ar id', 'ar it',
                 'ar ja', 'ar kk', 'ar nl', 'ar pt', 'ar ru', 'ar zh', 'cs de', 'cs en', 'cs es',
                 'cs fr', 'cs hi', 'cs id', 'cs it', 'cs ja', 'cs kk', 'cs nl', 'cs pt', 'cs ru',
                 'cs zh', 'de en', 'de es', 'de fr', 'de hi', 'de id', 'de it', 'de ja', 'de kk',
                 'de nl', 'de pt', 'de ru', 'de zh', 'en es', 'en fr', 'en hi', 'en id', 'en it',
                 'en ja', 'en kk', 'en nl', 'en pt', 'en ru', 'en zh', 'es fr', 'es hi', 'es id',
                 'es it', 'es ja', 'es kk', 'es nl', 'es pt', 'es ru', 'es zh', 'fr hi', 'fr id',
                 'fr it', 'fr ja', 'fr kk', 'fr nl', 'fr pt', 'fr ru', 'fr zh', 'hi id', 'hi it',
                 'hi nl', 'hi pt', 'hi ru', 'hi zh', 'id it', 'id kk', 'id nl', 'id pt', 'id ru',
                 'id zh', 'it kk', 'it nl', 'it pt', 'it ru', 'it zh', 'ja ru', 'ja zh', 'kk nl',
                 'kk pt', 'kk ru', 'kk zh', 'nl pt', 'nl ru', 'nl zh', 'pt ru', 'pt zh', 'ru zh']:
        l1, l2 = pair.split()
        index.add_entry(Entry(
            did=DatasetId(group=group_id, name='news_commentary', version='14', langs=(l1, l2)),
            url=NEWSCOM_v14 % (l1, l2), cite=cite))

    for v in [15, 16]:
        cite = index.ref_db.get_bibtex('barrault-etal-2020-findings')
        url = f"http://data.statmt.org/news-commentary/v{v}/training/news-commentary-v{v}.%s-%s.tsv.gz"
        for pair in ['ar cs', 'ar de', 'ar en', 'ar es', 'ar fr', 'ar hi', 'ar id', 'ar it', 'ar ja', 'ar kk', 'ar nl',
                     'ar pt', 'ar ru', 'ar zh', 'cs de', 'cs en', 'cs es', 'cs fr', 'cs hi', 'cs id', 'cs it', 'cs ja',
                     'cs kk', 'cs nl', 'cs pt', 'cs ru', 'cs zh', 'de en', 'de es', 'de fr', 'de hi', 'de id', 'de it',
                     'de ja', 'de kk', 'de nl', 'de pt', 'de ru', 'de zh', 'en es', 'en fr', 'en hi', 'en id', 'en it',
                     'en ja', 'en kk', 'en nl', 'en pt', 'en ru', 'en zh', 'es fr', 'es hi', 'es id', 'es it', 'es ja',
                     'es kk', 'es nl', 'es pt', 'es ru', 'es zh', 'fr hi', 'fr id', 'fr it', 'fr ja', 'fr kk', 'fr nl',
                     'fr pt', 'fr ru', 'fr zh', 'hi id', 'hi it', 'hi nl', 'hi pt', 'hi ru', 'hi zh', 'id it', 'id ja',
                     'id kk', 'id nl', 'id pt', 'id ru', 'id zh', 'it kk', 'it nl', 'it pt', 'it ru', 'it zh', 'ja pt',
                     'ja ru', 'ja zh', 'kk nl', 'kk pt', 'kk ru', 'kk zh', 'nl pt', 'nl ru', 'nl zh', 'pt ru', 'pt zh',
                     'ru zh']:
            l1, l2 = pair.split()
            index.add_entry(Entry(did=DatasetId(group=group_id, name='news_commentary', version=f'{v}', langs=(l1, l2)),
                url=url % (l1, l2), cite=cite))


    # ===== Wiki Titles V1
    WIKI_TITLES_v1 = 'http://data.statmt.org/wikititles/v1/wikititles-v1.%s-%s.tsv.gz'
    cite = index.ref_db.get_bibtex('barrault-etal-2019-findings')
    for pair in ['cs en', 'cs pl', 'de en', 'es pt', 'fi en', 'gu en', 'hi ne', 'kk en', 'lt en',
                 'ru en', 'zh en']:
        l1, l2 = pair.split()
        index.add_entry(Entry(did=DatasetId(group=group_id, name='wiki_titles', version='1', langs=(l1, l2)),
                              url=WIKI_TITLES_v1 % (l1, l2), cite=cite))

    # ===== Wiki Titles V2
    WIKI_TITLES_v2 = 'http://data.statmt.org/wikititles/v2/wikititles-v2.%s-%s.tsv.gz'
    for pair in ['ca es', 'cs en', 'de en', 'de fr', 'es pt', 'iu en', 'ja en', 'pl en', 'ps en',
                 'ru en', 'ta en', 'zh en']:
        l1, l2 = pair.split()
        index.add_entry(Entry(did=DatasetId(group=group_id, name='wiki_titles', version='2', langs=(l1, l2)),
                              url=WIKI_TITLES_v2 % (l1, l2), cite=cite))

    # ==== WMT  Dev and Tests
    wmt_sets = {
        'newstest2014': [('de', 'en'), ('cs', 'en'), ('fr', 'en'), ('ru', 'en'), ('hi', 'en')],
        'newsdev2015': [('fi', 'en'), ('en', 'fi')],
        'newstest2015': [('fi', 'en'), ('en', 'cs'), ('cs', 'en'), ('en', 'ru'), ('en', 'de'),
                         ('de', 'en'), ('ru', 'en'), ('en', 'fi')],
        'newsdev2016': [('en', 'ro'), ('ro', 'en'), ('tr', 'en'), ('en', 'tr')],
        'newstest2016': [('de', 'en'), ('en', 'de'), ('en', 'ro'), ('en', 'fi'), ('ro', 'en'),
                         ('ru', 'en'), ('fi', 'en'), ('en', 'ru'), ('tr', 'en'), ('cs', 'en'),
                         ('en', 'tr'), ('en', 'cs')],
        'newsdev2017': [('zh', 'en'), ('lv', 'en'), ('en', 'zh'), ('en', 'lv')],
        'newstest2017': [('zh', 'en'), ('ru', 'en'), ('en', 'fi'), ('lv', 'en'), ('en', 'de'),
                         ('de', 'en'), ('cs', 'en'), ('en', 'cs'), ('en', 'tr'), ('en', 'ru'),
                         ('tr', 'en'), ('fi', 'en'), ('en', 'zh'), ('en', 'lv')],
        'newsdev2018': [('et', 'en'), ('en', 'et')],
        'newstest2018': [('ru', 'en'), ('zh', 'en'), ('et', 'en'), ('en', 'fi'), ('en', 'de'),
                         ('de', 'en'), ('en', 'cs'), ('en', 'tr'), ('cs', 'en'), ('tr', 'en'),
                         ('en', 'ru'), ('en', 'et'), ('fi', 'en'), ('en', 'zh')],
        'newsdev2019': [('gu', 'en'), ('kk', 'en'), ('en', 'lt'), ('en', 'kk'), ('lt', 'en'),
                        ('en', 'gu')],
        'newstest2019': [('de', 'en'), ('de', 'fr'), ('kk', 'en'), ('en', 'de'), ('en', 'fi'),
                         ('ru', 'en'), ('zh', 'en'), ('gu', 'en'), ('en', 'kk'), ('en', 'zh'),
                         ('cs', 'de'), ('fi', 'en'), ('en', 'gu'), ('lt', 'en'), ('de', 'cs'),
                         ('en', 'lt'), ('en', 'ru'), ('en', 'cs'), ('fr', 'de')],
        'newsdev2020': [('iu', 'en'), ('en', 'ta'), ('ta', 'en'), ('pl', 'en'), ('en', 'iu'),
                        ('en', 'ja'), ('ja', 'en'), ('en', 'pl')]
    }
    for set_name, pairs in wmt_sets.items():
        sub_name, year = set_name[:-4], set_name[-4:]
        for l1, l2 in pairs:
            src = f'dev/{set_name}-{l1}{l2}-src.{l1}.sgm'
            ref = f'dev/{set_name}-{l1}{l2}-ref.{l2}.sgm'
            name = f'{sub_name}_{l1}{l2}'
            index.add_entry(Entry(did=DatasetId(group=group_id, name=name, version=year, langs=(l1, l2)),
                                  filename='wmt20dev.tgz', in_paths=[src, ref], in_ext='sgm',
                                  url='http://data.statmt.org/wmt20/translation-task/dev.tgz',
                                  cite=cite))

    # Multi parallel
    wmt_sets = {
        '2009': ['en', 'cs', 'de', 'es', 'fr'],
        '2010': ['en', 'cs', 'de', 'es', 'fr'],
        '2011': ['en', 'cs', 'de', 'es', 'fr'],
        '2012': ['en', 'cs', 'de', 'es', 'fr', 'ru'],
        '2013': ['en', 'cs', 'de', 'es', 'fr', 'ru'],
    }
    for year, langs in wmt_sets.items():
        name = 'newstest'
        for l1, l2 in itertools.combinations(langs, 2):
            f1 = f'dev/{name}{year}.{l1}'
            f2 = f'dev/{name}{year}.{l2}'
            index.add_entry(Entry(did=DatasetId(group=group_id, name=name, version=year, langs=(l1, l2)),
                                    filename='wmt20dev.tgz', in_paths=[f1, f2], in_ext='txt', cite=cite,
                                  url='http://data.statmt.org/wmt20/translation-task/dev.tgz'))

    for l1, l2 in [('ps', 'en'), ('km', 'en')]:
        for set_name in ['wikipedia.dev', 'wikipedia.devtest']:
            src = f'dev/{set_name}.{l1}-{l2}.{l1}'
            ref = f'dev/{set_name}.{l1}-{l2}.{l2}'
            name = f'{set_name.replace(".", "_")}_{l1}{l2}'
            index.add_entry(Entry(did=DatasetId(group=group_id, name=name, version='1', langs=(l1, l2)),
                                 filename='wmt20dev.tgz', in_paths=[src, ref], in_ext='txt', cite=cite,
                                  url='http://data.statmt.org/wmt20/translation-task/dev.tgz'))

    #### WMT 20 Tests
    url = "http://data.statmt.org/wmt20/translation-task/test.tgz"
    wmt20_cite = index.ref_db.get_bibtex('barrault-etal-2020-findings')
    for _pref, pairs in {
        "": ["csen", "deen", "defr", "encs", "ende", "eniu", "enja", "enkm", "enpl", "enps",
             "enru", "enta", "enzh", "frde", "iuen", "jaen", "kmen", "plen", "psen", "ruen",
             "taen", "zhen"],
        "B": ["deen", "ende", "enzh", "ruen", "zhen"]}.items():
        year = "2020"
        name = f'newstest{_pref}'
        for pair in pairs:
            l1, l2 = pair[:2], pair[2:]
            f1 = f'sgm/{name}{year}-{pair}-src.{l1}.sgm'
            f2 = f'sgm/{name}{year}-{pair}-ref.{l2}.sgm'
            index.add_entry(Entry(did=DatasetId(group=group_id, name=f'{name}_{pair}'.lower(), version=year, langs=(l1, l2)),
                filename='wmt20tests.tgz', in_paths=[f1, f2], in_ext='sgm', cite=wmt20_cite, url=url))

    # WMT 21 Dev
    url = "http://data.statmt.org/wmt21/translation-task/dev.tgz"
    pairs = "en-ha en-is is-en ha-en".split()
    for pair in pairs:
        l1, l2 = pair.split('-')
        in_path = f'dev/xml/newsdev2021.{l1}-{l2}.xml'
        ent = Entry(did=DatasetId(group=group_id, name=f'newsdev_{l1}{l2}', version='2021', langs=(l1, l2)),
                    filename='wmt21dev.tgz', in_paths=[in_path], in_ext='wmt21xml', cite=wmt20_cite, url=url)
        index.add_entry(ent)


    # ==== TED Talks 2.0 ar-en
    index.add_entry(Entry(did=DatasetId(group=group_id, name='tedtalks', version='2_clean', langs=('en', 'ar')),
                         ext='tsv.xz', url='http://data.statmt.org/ted-talks/en-ar.v2.aligned.clean.xz'))

    # ==== Europarl v10
    EP_v10 = "http://www.statmt.org/europarl/v10/training/europarl-v10.%s-%s.tsv.gz"
    for pair in ['cs en', 'cs pl', 'de en', 'de fr', 'es pt', 'fi en', 'fr en', 'lt en', 'pl en']:
        l1, l2 = pair.split()
        index.add_entry(Entry(did=DatasetId(group=group_id, name=f'europarl', version='10', langs=(l1, l2)),
                url=EP_v10 % (l1, l2), cite=wmt20_cite))

    # ==== PMIndia V1
    PMINDIA_v1 = "http://data.statmt.org/pmindia/v1/parallel/pmindia.v1.%s-%s.tsv"
    cite = index.ref_db.get_bibtex('Haddow-etal-2020-PMIndia')
    for pair in ["as en", "bn en", "gu en", "hi en", "kn en", "ml en", "mni en", "mr en", "or en",
                 "pa en", "ta en", "te en", "ur en"]:
        l1, l2 = pair.split()
        # Note: listed as xx-en in URL but actually en-xx in the tsv; and its not compressed!
        index.add_entry(Entry(did=DatasetId(group=group_id, name=f'pmindia', version='1', langs=(l2, l1)),
                              url=PMINDIA_v1 % (l1, l2), cite=cite))

    # Pashto - English  pseudo parallel dataset for alignment
    index.add_entry(Entry(did=DatasetId(group=group_id, name=f'wmt20_enps_aligntask', version='1', langs=('en', 'ps')),
                          url='http://data.statmt.org/wmt20/translation-task/ps-km/wmt20-sent.en-ps.xz',
                          cite=wmt20_cite, ext='tsv.xz'))

    # Pashto - English  mostly parallel dataset
    for name in ["GNOME.en-ps", "KDE4.en-ps", "Tatoeba.en-ps", "Ubuntu.en-ps", "bible.en-ps.clean",
                 "ted-wmt20.en-ps", "wikimedia.en-ps"]:
        ps = f'ps-parallel/{name}.ps'
        en = f'ps-parallel/{name}.en'
        url = 'http://data.statmt.org/wmt20/translation-task/ps-km/ps-parallel.tgz'
        name = name.replace('.en-ps', '').replace('.', '_').replace('-', '_').lower()
        entry = Entry(did=DatasetId(group=group_id, name=name, version='1', langs=('ps', 'en')), url=url,
                      cite=wmt20_cite, in_paths=[ps, en], filename='wmt20-psen-parallel.tgz', in_ext='txt')
        index.add_entry(entry)

    for l2 in ['ps', 'km']:
        url = f"http://data.statmt.org/wmt20/translation-task/ps-km/wmt20-sent.en-{l2}.xz"
        entry = Entry(did=DatasetId(group=group_id, name='paracrawl', version='5.1', langs=('en', l2)),
                    url=url, cite=wmt20_cite, ext='tsv.xz', cols=(0, 1))
        index.add_entry(entry)

    ccalign_cite = index.ref_db.get_bibtex('chaudhary-EtAl:2019:WMT')
    CC_ALIGNED = 'http://www.statmt.org/cc-aligned/sentence-aligned/{src}-{tgt}.tsv.xz'
    tgts='es_XX et_EE fa_IR ff_NG fi_FI fr_XX gu_IN ha_NG he_IL hi_IN hr_HR ht_HT hu_HU hy_AM id_ID ig_NG is_IS it_IT ja_XX jv_ID ka_GE kg_AO kk_KZ km_KH kn_IN ko_KR ku_TR ky_KG lg_UG ln_CD lo_LA lt_LT lv_LV mg_MG mi_NZ mk_MK ml_IN mn_MN mr_IN ms_MY mt_MT my_MM ne_NP nl_XX no_XX ns_ZA ny_MW om_KE or_IN pa_IN pl_PL ps_AF pt_XX qa_MM qd_MM ro_RO ru_RU si_LK sk_SK sl_SI sn_ZW so_SO sq_AL sr_RS ss_SZ st_ZA su_ID sv_SE sw_KE sz_PL ta_IN te_IN tg_TJ th_TH ti_ET tl_XX tn_BW tr_TR ts_ZA tz_MA uk_UA ur_PK ve_ZA vi_VN wo_SN xh_ZA yo_NG zh_CN zh_TW zu_ZA zz_TR'.split()
    srcs = 'af_ZA ak_GH am_ET ar_AR as_IN ay_BO az_AZ az_IR be_BY bg_BG bm_ML bn_IN br_FR bs_BA ca_ES cb_IQ cs_CZ cx_PH cy_GB da_DK de_DE el_GR'.split()
    pairs = [('en_XX', tgt) for tgt in tgts] + [(src, 'en_XX') for src in srcs]
    dont_know = {'qa', 'qd'}   # looks like some Myanmar languages, but not sure which one.
    # Cant find them in ISO 639-1:  https://en.wikipedia.org/wiki/List_of_ISO_639-1_codes
    #                and lingo http://www.lingoes.net/en/translator/langcode.htm
    #               and web-info https://wp-info.org/tools/languagecodes.php
    #unsupported = {'zh_TW', 'az_IR'}
    # country locales are not supported; they create conflicts. keeping large ones instead
    for src, tgt in pairs:
        # l1, l2 = src.split('_')[0], tgt.split('_')[0]
        if src[:2] in dont_know or tgt[:2] in dont_know:   # I dont know what language these are
            continue
        url = CC_ALIGNED.format(src=src, tgt=tgt)
        entry = Entry(did=DatasetId(group=group_id, name='ccaligned', version='1', langs=(src, tgt)), url=url,
                      cite=ccalign_cite, ext='tsv.xz', cols=(0, 1))
        index.add_entry(entry)



#!/usr/bin/env python
#
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 4/8/20

from typing import Tuple, List, Optional, Set, Union
from dataclasses import dataclass, field
from mtdata.iso.bcp47 import BCP47Tag, bcp47

DID_DELIM = '-'  # I  wanted to use ":", but Windows, they dont like ":" in path! :(

LangPair = Tuple[BCP47Tag, BCP47Tag]


@dataclass(frozen=True)
class DatasetId:
    group: str
    name: str
    version: str
    langs: Union[Tuple[str, str], LangPair]  # one=monolingual, two=bitext; many=multi

    def __post_init__(self):
        assert self.group
        assert self.name
        assert self.version
        assert self.name.islower(), f'name {self.name} has to be lower cased for consistency'
        for name in [self.group, self.version, self.name]:
            for ch in '-/*|[](){}<>?&:;,!^$"\' ':
                assert ch not in name, f"Character '{ch}' is not permitted in name {name}"
        # ensure lang ID is BCP47 tag
        assert isinstance(self.langs, tuple), f'Expected tuple (l1, l2); given={self.langs}'
        langs = tuple(lang if isinstance(lang, BCP47Tag) else bcp47(lang) for lang in self.langs)
        if langs != self.langs:
            object.__setattr__(self, 'langs', langs)  # bypass frozen=True

    @property
    def lang_str(self):
        return DID_DELIM.join(str(lang) for lang in self.langs)

    def format(self, delim=DID_DELIM):
        return delim.join([self.group, self.name, self.version, self.lang_str])

    def __str__(self):
        return self.format()

    @classmethod
    def parse(cls, string, delim=DID_DELIM) -> 'DatasetId':
        expected_format = f"<group>{delim}<name>{delim}<version>{delim}<l1>{delim}<l2>"
        parts = string.strip().split(delim)
        if len(parts) != 5:
            raise Exception(f'Dataset ID expected in format: {expected_format}; but given {string}.'
                            f' If you are unsure, run "mtdata list | cut -f1 | grep -i <name>" and copy its id.')
        group, name, version, lang1, lang2 = parts
        return cls(group=group, name=name, version=version, langs=(lang1, lang2))


class Entry:
    __slots__ = ('did', 'url', 'filename', 'ext', 'in_paths', 'in_ext', 'cite', 'cols', 'is_archive')

    def __init__(self, did: DatasetId,
                 url: str,
                 filename: Optional[str] = None,
                 ext: Optional[str] = None,
                 in_paths: Optional[List[str]] = None,
                 in_ext: Optional[str] = None,
                 cite: Optional[str] = None,
                 cols: Optional[Tuple[int, int]] = None):

        assert isinstance(did, DatasetId)
        self.did = did
        self.url = url
        self.filename = filename
        orig_name = self.url.split('/')[-1]
        self.ext = ext
        if not self.ext:
            from mtdata.parser import detect_extension
            self.ext = detect_extension(filename or orig_name)
        self.filename = self.filename or f'{self.did.name}.{self.ext}'

        self.in_paths = in_paths
        self.in_ext = in_ext
        self.cite = cite
        self.cols = cols

        assert not self.ext.startswith("."), f'ext {self.ext} should not start with a dot (.)'
        self.is_archive = self.ext in ('zip', 'tar', 'tar.gz', 'tgz')
        if self.is_archive:
            assert self.in_paths and len(self.in_paths) > 0, 'Archive entries must have in_paths'
            if not self.in_ext:
                raise Exception('in_ext is required for archive files')
        else:
            if self.in_ext != 'opus_xces':
                assert not self.in_paths, f"in_paths is not applicable for non archive format {self.ext}"

    def is_swap(self, langs):
        if self.in_ext == 'tmx':
            return False
        return tuple(reversed(langs)) == tuple(self.lang_str)

    def __str__(self):
        return self.format(delim=' ')

    @property
    def lang_str(self):
        return self.did.lang_str

    def format(self, delim: str = ' '):
        msg = f'{self.did}{delim}{self.url}{delim}{",".join(self.in_paths or [])}'
        return msg

    def is_noisy(self, seg1, seg2) -> bool:
        # None or Empty
        noisy = seg1 is None or seg2 is None or not seg1.strip() or not seg2.strip()
        return noisy


class JW300Entry(Entry):
    url: Tuple[str, str, str]  # (align.xml, src.xml, tgt.xml)


@dataclass
class Experiment:
    langs: Tuple[BCP47Tag, BCP47Tag]  # (lang1 , lang2)  lang1 -> lang2
    train: List[Entry]  # training should be merged from all these
    tests: List[Entry]  # multiple tests; one of them can be validation set
    papers: Set['Paper'] = field(default_factory=set)

    def __post_init__(self):
        if any(not isinstance(lang, BCP47Tag) for lang in self.langs):
            self.langs = tuple(bcp47(l) for l in self.langs)
        for t in self.tests:
            assert t
        for t in self.train:
            assert t

    @classmethod
    def make(cls, index, langs: Tuple[str, str], train: List[str], tests: List[str]):
        train = [index.get_entry(name, langs) for name in train]
        tests = [index.get_entry(name, langs) for name in tests]
        return cls(langs, train=train, tests=tests)


@dataclass(eq=False)  # see for hash related issues: https://stackoverflow.com/a/52390734/1506477
class Paper:  # or Article

    name: str  # author1-etal-year
    title: str  # title
    url: str  # Paper url to be sure
    cite: str  # bibtex would be nice to display
    experiments: List[Experiment]

    langs: Set[Tuple[str, str]] = None

    def __post_init__(self):
        self.langs = self.langs or set(exp.langs for exp in self.experiments)
        for exp in self.experiments:
            exp.papers.add(self)



#!/usr/bin/env python
#
#
# Author: Thamme Gowda
# Created: 10/4/21
from mtdata.iso.bcp47 import bcp47, BCP47Tag
from pytest import fail


def test_bcp47():
    assert bcp47("en-GB")[:3] == ('eng', None, 'GB')
    assert bcp47("en-GB") == ('eng', None, 'GB', 'eng_GB')
    assert bcp47("en") == ('eng', None, None, 'eng')
    assert bcp47("en-IN") == ('eng', None, 'IN', 'eng_IN')
    assert bcp47("en-US") == ('eng', None, 'US', 'eng_US')
    # Latn script is default, so None
    assert bcp47("en-Latn") == ('eng', None, None, 'eng')
    # hypothetical;
    assert bcp47("en-Knda") == ('eng', 'Knda', None, 'eng_Knda')
    assert bcp47("en-Latn-GB") == ('eng', None, 'GB', 'eng_GB')

    try:
        bcp47("en-Latn-UK")
        fail("UK is not ISO country code")
    except ValueError:
        pass  # expected

    assert bcp47("kn") == ('kan', None, None, 'kan')
    assert bcp47("kn-Knda") == ('kan', None, None, 'kan')    # default script
    assert bcp47("kn-Knda-IN") == ('kan', None, 'IN', 'kan_IN')
    assert bcp47("kn_Knda_IN") == ('kan', None, 'IN', 'kan_IN')
    assert bcp47("kn_Knda_in") == ('kan', None, 'IN', 'kan_IN')
    assert bcp47("kn_Latn_IN") == ('kan', 'Latn', 'IN', 'kan_Latn_IN')
    assert bcp47("kn_Deva_IN") == ('kan', 'Deva', 'IN', 'kan_Deva_IN')
    assert bcp47("hi_Deva_IN") == ('hin', None, 'IN', 'hin_IN')    # default script

    assert bcp47("pt_PT") == ('por', None, 'PT', 'por_PT')
    assert bcp47("pt_pt") == ('por', None, 'PT', 'por_PT')
    assert bcp47("pt_BR") == ('por', None, 'BR', 'por_BR')
    assert bcp47("pt_br") == ('por', None, 'BR', 'por_BR')
    assert bcp47("pt_Latn_br") == ('por', None, 'BR', 'por_BR')  # default script
    assert bcp47("pt_Cyrl_br") == ('por', 'Cyrl', 'BR', 'por_Cyrl_BR')  # non default script
    assert bcp47("fr") == ('fra', None, None, 'fra')
    assert bcp47("fr-CA") == ('fra', None, 'CA', 'fra_CA')


def test_py_obj_model():
    """Test cases for python object model"""
    assert bcp47("en-GB") is not bcp47("en_GB")  # object comparison: two objects are two different references
    assert bcp47("en-GB") == bcp47("en_GB")      # but they have same value and hence equal
    assert bcp47("en-GB") == bcp47("en_Latn_GB")  # ignore the default script
    assert bcp47("en-Latn-US") == bcp47("en_US")  # ignore default script again
    assert bcp47("en-Latn") == bcp47("en")  # ignore default script again
    assert bcp47("en-Latn") == bcp47("english")  # ignore default script again
    assert bcp47("en-Latn") == bcp47("English")  # ignore default script again
    assert bcp47("en-Latn") == bcp47("ENG")  # ignore default script again
    assert bcp47("en-US") != bcp47("en")      # dont ignore region

    # Custom class, e.g. BCP47Tag, instead of plain old str obj could create bugs due to improper hashing;
    # so test it out
    mem = set()
    assert bcp47('en') not in mem
    mem.add(bcp47('en'))
    assert bcp47('en') in mem
    mem.add(bcp47('en'))
    assert len(mem) == 1  # dupes are removed
    mem.add(bcp47('en-Latn'))
    mem.add(bcp47('english'))
    mem.add(bcp47('English'))
    mem.add(bcp47('eng'))
    assert len(mem) == 1       # dupes are removed

    mem = dict()
    mem[bcp47('en')] = 10
    assert mem[bcp47('en')] == 10
    assert mem[bcp47('en')] != 11
    assert mem[bcp47('eng')] == 10
    assert mem[bcp47('english')] == 10
    assert mem[bcp47('English-Latn')] == 10




#!/usr/bin/env python
#
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 4/8/20

from mtdata.index import Entry, Index, DatasetId


def load(index: Index):
    cite = index.ref_db.get_bibtex('espla-etal-2019-paracrawl')
    cite += '\n' + index.ref_db.get_bibtex('banon-etal-2020-paracrawl')
    group_id = 'ParaCrawl'
    # === Para crawl corpus
    PARACRAWL_v3 = 'https://s3.amazonaws.com/web-language-models/paracrawl/release3/%s-%s.bicleaner07.tmx.gz'
    for pair in ['en cs', 'en de', 'en fi', 'en lt']:
        l1, l2 = pair.split()
        index.add_entry(Entry(did=DatasetId(group=group_id, name=f'paracrawl', version='3', langs=(l1, l2)),
                              url=PARACRAWL_v3 % (l1, l2), cite=cite))

    # === Paracrawl V6
    PARACRAWL_v6 = 'https://s3.amazonaws.com/web-language-models/paracrawl/release6/%s-%s.txt.gz'
    for l2 in ['is', 'bg', 'hr', 'cs', 'da', 'nl', 'et', 'fi', 'fr', 'de', 'el', 'hu', 'ga', 'it', 'lv',
               'lt', 'mt', 'pl', 'pt', 'ro', 'sk', 'sl', 'es', 'sv']:
        l1 = 'en'
        index.add_entry(Entry(did=DatasetId(group=group_id, name=f'paracrawl', version='6', langs=(l1, l2)),
                              url=PARACRAWL_v6 % (l1, l2), cite=cite, ext='tsv.gz'))
    # these are bonus
    PARACRAWL_v6_B = 'https://s3.amazonaws.com/web-language-models/paracrawl/release6/%s-%s.bicleaner07.txt.gz'
    for l1, l2 in [('nl', 'fr'), ('pl', 'de')]:
        index.add_entry(Entry(did=DatasetId(group=group_id, name=f'paracrawl', version='6B', langs=(l1, l2)),
                              url=PARACRAWL_v6_B % (l1, l2), cite=cite, ext='tsv.gz'))

    l1 = 'en'
    PARACRAWL_v7_1 = 'https://s3.amazonaws.com/web-language-models/paracrawl/release7.1/%s-%s.txt.gz'
    for l2 in 'bg cs da de el es et fi fr ga hr hu is it lt lv mt nl pl pt ro sk sl sv'.split():
        index.add_entry(Entry(did=DatasetId(group=group_id, name=f'paracrawl', version='7.1', langs=(l1, l2)),
                              url=PARACRAWL_v7_1 % (l1, l2), cite=cite, ext='tsv.gz'))
    PARACRAWL_v7_1 = 'https://s3.amazonaws.com/web-language-models/paracrawl/release7/%s-%s.txt.gz'
    for pair in 'en-nb en-nn es-ca es-eu es-gl'.split():
        l1, l2 = pair.split('-')
        index.add_entry(Entry(did=DatasetId(group=group_id, name=f'paracrawl', version='7.1', langs=(l1, l2)),
                              url=PARACRAWL_v7_1 % (l1, l2), cite=cite, ext='tsv.gz'))

    PARACRAWL_V8 = 'https://archive.org/download/ParaCrawl-{version}/{pair}.txt.gz'
    for version, pairs in [
        ('v8.0', 'en-bg en-cs en-da en-de en-el'),
        ('v8.0-0001',
         'en-et en-fi en-fr en-ga en-hr en-hu en-is en-it en-lt en-lv en-mt en-nl en-no en-pl en-pt en-ro en-sk en-sl'),
        ('v8.0-0002', 'en-sv es-eu'),
        ('v8.1-0000', 'es-ca es-gl')]:
        for pair in pairs.split():
            l1, l2 = pair.split('-')
            url = PARACRAWL_V8.format(version=version, pair=pair)
            ent = Entry(did=DatasetId(group=group_id, name=f'paracrawl', version='8', langs=(l1, l2)),
                        url=url, cite=cite, ext='tsv.gz')
            index.add_entry(ent)

    PARACRAWL_BONUS = 'https://s3.amazonaws.com/web-language-models/paracrawl/bonus/{pair}.txt.gz'
    for pair in 'en-km en-my en-ne en-ps en-si en-so en-sw en-tl en-ru en-ko'.split():
        l1, l2 = pair.split('-')
        url = PARACRAWL_BONUS.format(pair=pair)
        ent = Entry(did=DatasetId(group=group_id, name=f'paracrawl', version='1_bonus', langs=(l1, l2)),
                    url=url, cite=cite, ext='tsv.gz')
        index.add_entry(ent)

    PARACRAWL_V9 = 'https://s3.amazonaws.com/web-language-models/paracrawl/release9/{l1}-{l2}/{l1}-{l2}.txt.gz'
    for pair in ('en-bg en-cs en-da en-de en-el en-es en-et en-fi en-fr en-ga en-hr en-hu en-is en-it en-lt en-lv'
                 ' en-mt en-nb en-nl en-nn en-pl en-pt en-ro en-sk en-sl en-sv es-ca es-eu es-gl').split():
        l1, l2 = pair.split('-')
        url = PARACRAWL_V9.format(l1=l1, l2=l2)
        ent = Entry(did=DatasetId(group=group_id, name=f'paracrawl', version='9', langs=(l1, l2)),
                    url=url, cite=cite, ext='tsv.gz')
        index.add_entry(ent)
    # this is a new addition in Sept 2021
    index.add_entry(Entry(
        did=DatasetId(group=group_id, name=f'paracrawl', version='1_bonus', langs=('en', 'zh')),
        url='http://web-language-models.s3-website-us-east-1.amazonaws.com/paracrawl/bonus/en-zh-v1.txt.gz',
        cite=cite, ext='tsv.gz'))

    # Japanese-English paracrawl (5.1) used by WMT20 and WMT21
    ent = Entry(did=DatasetId(group='KECL', name=f'paracrawl', version='2', langs=('eng', 'jpn')),
                in_paths=['en-ja/en-ja.bicleaner05.txt'], in_ext='tsv', cols=(2, 3), cite='',
                url='http://www.kecl.ntt.co.jp/icl/lirg/jparacrawl/release/2.0/bitext/en-ja.tar.gz')
    index.add_entry(ent)



#!/usr/bin/env python
#
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 4/4/20
import zipfile
import tarfile
import fnmatch
from dataclasses import dataclass
from pathlib import Path
from mtdata.index import Entry
from mtdata import log, __version__, pbar_man, MTDataException
from mtdata.utils import ZipPath, TarPath
from typing import List, Union

import portalocker
from hashlib import md5
from urllib.parse import urlparse
from .parser import detect_extension
import requests
import math

MAX_TIMEOUT = 2 * 60 * 60  # 2 hours

headers = {'User-Agent': f'mtdata downloader {__version__}; cURL and wget like.'}


@dataclass
class Cache:
    root: Path

    def __post_init__(self):
        if isinstance(self.root, str):
            self.root = Path(self.root)
        log.info(f"Local cache is at {self.root}")

    def get_entry(self, entry: Entry, fix_missing=True) -> Union[Path, List[Path]]:
        if entry.in_ext == 'opus_xces':
            return self.opus_xces_format(entry=entry, fix_missing=fix_missing)

        local = self.get_local_path(entry.url, filename=entry.filename, fix_missing=fix_missing)
        if zipfile.is_zipfile(local) or tarfile.is_tarfile(local):
            # look inside the archives and get the desired files
            local = self.get_local_in_paths(path=local, entry=entry)
        return local

    def get_flag_file(self, file: Path):
        return file.with_name(file.name + '._valid')

    def get_local_path(self, url, filename=None, fix_missing=True):
        hostname = urlparse(url).hostname or 'nohost'
        filename = filename or url.split('/')[-1]
        assert hostname and filename
        mdf5_sum = md5(url.encode('utf-8')).hexdigest()
        local = self.root / hostname / mdf5_sum[:4] / mdf5_sum[4:] / filename
        if fix_missing:
            self.download(url, local)
        return local

    @classmethod
    def match_globs(cls, names, globs, meta=''):
        result = []
        for pat in globs:
            matches = fnmatch.filter(names, pat)
            if len(matches) != 1:
                raise MTDataException(f'{meta} {pat} matched {matches}; expected one file')
            result.append(matches[0])
        return result

    def opus_xces_format(self, entry, fix_missing=True) -> List[Path]:
        assert entry.in_ext == 'opus_xces'
        l1_url, l2_url = entry.in_paths
        align_file = self.get_local_path(entry.url, fix_missing=fix_missing)
        l1_path = self.get_local_path(l1_url, fix_missing=fix_missing)
        l2_path = self.get_local_path(l2_url, fix_missing=fix_missing)
        return [align_file, l1_path, l2_path]

    def get_local_in_paths(self, path:Path, entry: Entry,):
        in_paths = entry.in_paths
        if zipfile.is_zipfile(path):
            with zipfile.ZipFile(path) as root:
                in_paths = self.match_globs(names=root.namelist(), globs=in_paths)
            return [ZipPath(path, p) for p in in_paths]   # stdlib is buggy, so I made a workaround
        elif tarfile.is_tarfile(path):
            with tarfile.open(path, encoding='utf-8') as root:
                in_paths = self.match_globs(names=root.getnames(), globs=in_paths)
            return [TarPath(path, p) for p in in_paths]
        else:
            raise Exception(f'Unable to read {entry.did}; the file is neither zip nor tar')

    def download(self, url: str, save_at: Path):
        valid_flag = self.get_flag_file(save_at)
        lock_file = valid_flag.with_suffix("._lock")
        if valid_flag.exists() and save_at.exists():
            return save_at
        save_at.parent.mkdir(parents=True, exist_ok=True)

        log.info(f"Acquiring lock on {lock_file}")
        with portalocker.Lock(lock_file, 'w', timeout=MAX_TIMEOUT) as fh:
            # check if downloaded by  other parallel process
            if valid_flag.exists() and save_at.exists():
                return save_at
            log.info(f"Downloading {url} --> {save_at}")
            resp = requests.get(url=url, allow_redirects=True, headers=headers, stream=True)
            assert resp.status_code == 200, resp.status_code
            buf_size = 2 ** 10
            n_buffers = math.ceil(int(resp.headers.get('Content-Length', '0')) / buf_size) or None
            desc = url
            if len(desc) > 40:
                desc = desc[:30] + '...' + desc[-10:]
            with pbar_man.counter(color='green', total=n_buffers, unit='KiB', leave=False,
                                  desc=f"{desc}") as pbar, open(save_at, 'wb', buffering=2**24) as out:
                for chunk in resp.iter_content(chunk_size=buf_size):
                    out.write(chunk)
                    pbar.update()
            valid_flag.touch()
            lock_file.unlink()
            return save_at


def right_replace(string, old, new):
    """
    replaces last occurrence of old with new in a string
    :param string:
    :param old:
    :param new:
    :return:
    """
    return new.join(string.rsplit(old))



from pathlib import Path
import csv

src_dir = Path(__file__).parent
ISO_3166_file = src_dir / 'iso3166.tsv'
ISO_15924_file = src_dir / 'iso15924.tsv'
ISO_639_3_file = src_dir / 'iso639_3.tsv'
IANA_tag_file = src_dir / 'language-subtag-registry'


def read_tsv(path: Path):
    assert path.exists(), f'file at {path} not found'
    with open(path, encoding='utf-8') as inp:
        yield from csv.reader(inp, delimiter='\t', quotechar=None)


def read_tsv_with_head(path: Path):
    rows = list(read_tsv(path))
    head, rows = rows[0], rows[1:]
    return head, rows


def load_country_codes():
    head, rows = read_tsv_with_head(ISO_3166_file)
    assert head[0] == 'English Name'
    assert head[2] == 'Alpha-2'

    def cleanup(name):
        return name.replace(' (the)', '')

    recs = [(r[2], cleanup(r[0]))for r in rows]
    return recs


def load_script_codes():
    head, rows = read_tsv_with_head(ISO_15924_file)
    assert head[0] == 'Code'
    assert head[2] == 'English Name'

    recs = [(r[0], r[2]) for r in rows]
    return recs


def load_language_codes():
    head, rows = read_tsv_with_head(ISO_639_3_file)
    assert head[0] == 'Id'
    assert head[3] == 'Part1'
    assert head[6] == 'Ref_Name'

    recs = [(r[0], r[3], r[6]) for r in rows]
    return recs


def parse_iana_tags(path: Path):

    def unwrap_lines(lines):
        """Some lines are wrapped; this mapper unwraps lines"""
        prev = ''
        wrap_prefix = '  '  # two spaces
        for i, line in enumerate(lines):
            if i == 0:
                prev = line
                continue
            if line.startswith(wrap_prefix):
                prev += ' ' + line.strip()
            else:
                yield prev
                prev = line
        if prev:
            yield prev

    def segment_recs(lines):
        marker = '%%'
        recs = []
        for line_num, line in enumerate(lines, start=1):
            line = line.strip()
            if line_num == 1 and 'File-Date:' in line:
                continue
            if line == marker:
                recs.append({})
            else:
                parts = line.split(":")
                assert len(parts) >= 2
                key = parts[0].strip()
                val = ':'.join(parts[1:]).strip()
                if key in recs[-1]:  # already exists
                    if not isinstance(recs[-1][key], list):
                        recs[-1][key] = [recs[-1][key]]
                    recs[-1][key].append(val)
                else:
                    recs[-1][key] = val
        return recs

    with open(path, encoding='utf-8') as lines:
        recs = segment_recs(unwrap_lines(lines))
    return recs


def load_iana_default_scripts():
    tags = parse_iana_tags(IANA_tag_file)
    scripts = []
    for tag in tags:
        if 'Suppress-Script' in tag:
            assert tag['Type'] == 'language', f'Error: {tag}'
            scripts.append([tag['Subtag'], tag['Suppress-Script'], tag['Description']])
    return scripts


def get_bcp47_data():
    return {
        'languages': load_language_codes(),
        'scripts': load_script_codes(),
        'countries': load_country_codes(),
        'default_scripts': load_iana_default_scripts()
    }


if __name__ == '__main__':
    import json
    data = get_bcp47_data()
    s = json.dumps(data, ensure_ascii=False)
    print(s.replace("],", "],\n  ").replace(": ", ":\n  ").replace('\n   "', '\n"'))
    #from pprint import pprint
    #pprint(data)




#!/usr/bin/env python
#
#
# Author: Thamme Gowda
# Created: 10/12/21

from mtdata.index import is_compatible, bcp47


def test_is_compatible():
    assert is_compatible(bcp47('en'), bcp47('en'))
    assert is_compatible(bcp47('en'), bcp47('en_US'))
    assert is_compatible(bcp47('en_US'), bcp47('en'))
    assert not is_compatible(bcp47('en_US'), bcp47('en_GB'))
    assert not is_compatible(bcp47('en_US'), bcp47('en_IN'))
    assert is_compatible(bcp47('en_US'), bcp47('en_Latn'))
    assert is_compatible(bcp47('en_US'), bcp47('en_Latn_US'))

    assert is_compatible(bcp47('por_BR'), bcp47('por'))
    assert is_compatible(bcp47('por_PT'), bcp47('por'))
    assert not is_compatible(bcp47('por_PT'), bcp47('por_BR'))

    assert is_compatible(bcp47('kan'), bcp47('kan_IN'))
    assert is_compatible(bcp47('kan'), bcp47('kan_Knda'))
    assert is_compatible(bcp47('kan'), bcp47('kan_Knda_IN'))
    assert not is_compatible(bcp47('kan'), bcp47('kan_Deva_IN'))
    assert not is_compatible(bcp47('hin_Deva_In'), bcp47('kan_Deva_IN'))
    assert not is_compatible(bcp47('hin_In'), bcp47('kan_Deva_IN'))
    assert not is_compatible(bcp47('hin'), bcp47('kan_Deva_IN'))



#!/usr/bin/env python
#
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 5/13/20
import io
import gzip
import tarfile
import zipfile
from dataclasses import dataclass

from mtdata import log
import shutil
from datetime import datetime
from pathlib import Path


class IO:
    """File opener and automatic closer
    Copied from my other project https://github.com/isi-nlp/rtg/blob/master/rtg/utils.py
    """
    def __init__(self, path, mode='r', encoding=None, errors=None, smart_ext=True):
        """

        Parameters
        ----------
        path: path or pathlke object
        mode: reader
        encoding
        errors
        smart_ext : enable compression when extension .xz or .gz are detected
        """
        self.smart_ext = smart_ext
        if hasattr(path, 'write') or hasattr(path, 'read'):
            self.path = None
            self.fd = path
        if hasattr(path, 'open'):  # pathlib.Path and zipfile.Path has open
            self.fd = None
            self.path = path
        else:
            self.fd = None
            self.path = Path(path)
        self.mode = mode
        self.encoding = encoding or 'utf-8' if 't' in mode else None
        self.errors = errors or 'replace'

    def __enter__(self):
        if not self.path and self.fd is not None:
            # already opened
            return self.fd

        if self.smart_ext and self.path.name.endswith(".gz"):   # gzip mode
            self.fd = gzip.open(self.path, self.mode, encoding=self.encoding, errors=self.errors)
        elif self.smart_ext and self.path.name.endswith(".xz"):
            import lzma
            self.fd = lzma.open(self.path, self.mode, encoding=self.encoding, errors=self.errors)
        else:
            if 'b' in self.mode:  # binary mode doesnt take encoding or errors
                self.fd = self.path.open(self.mode)
            else:
                self.fd = self.path.open(self.mode, encoding=self.encoding, errors=self.errors, newline='\n')
        return self.fd

    def __exit__(self, _type, value, traceback):
        self.fd.close()
        self.fd = None

    @classmethod
    def reader(cls, path, text=True, **kwargs):
        return cls(path, 'rt' if text else 'rb', **kwargs)

    @classmethod
    def writer(cls, path, text=True, append=False, **kwargs):
        return cls(path, ('a' if append else 'w') + ('t' if text else 'b'), **kwargs)

    @classmethod
    def get_lines(cls, path, col=0, delim='\t', line_mapper=None, newline_fix=True):
        with cls.reader(path) as inp:
            if newline_fix and delim != '\r':
                inp = (line.replace(b'\r', b'') for line in inp)
            if col >= 0:
                inp = (line.split(delim)[col].strip() for line in inp)
            if line_mapper:
                inp = (line_mapper(line) for line in inp)
            yield from inp

    @classmethod
    def get_liness(cls, *paths, **kwargs):
        for path in paths:
            yield from cls.get_lines(path, **kwargs)

    @classmethod
    def write_lines(cls, path: Path, text):
        if isinstance(text, str):
            text = [text]
        with cls.writer(path) as out:
            for line in text:
                out.write(line)
                out.write('\n')

    @classmethod
    def copy_file(cls, src: Path, dest: Path, text=False):
        assert src.resolve() != dest.resolve()
        log.info(f"Copy {src} → {dest}")
        with IO.reader(src, text=text) as inp, IO.writer(dest, text=text) as out:
            shutil.copyfileobj(inp, out)

    @classmethod
    def maybe_backup(cls, file: Path):
        if file.exists():
            time = datetime.now().strftime('%Y%m%d_%H%M%S_%f')
            dest = file.with_suffix(f'.{time}')
            log.info(f"Backup {file} → {dest}")
            file.rename(dest)


@dataclass
class ArchivedPath:

    root: Path
    name: str
    fd = None

    def __post_init__(self):
        assert self.root.exists()
        assert self.name

    def open(self, mode='r', **kwargs):
        raise NotImplemented


@dataclass
class ZipPath (ArchivedPath):
    # there is a bug in stdlib's zipfile.Path https://bugs.python.org/issue40564, so this is a workaround

    def __post_init__(self):
        assert zipfile.is_zipfile(self.root)

    def exists(self):
        with zipfile.ZipFile(self.root) as root:
            #  zipfile.Path is available in 3.8+
            return self.name in root.namelist()

    def open(self, mode='r', **kwargs):
        assert mode in ('r', 'rt'), f'only "r" is supported, given: {mode}'
        log.debug(f"Reading from zip file : {self.root} // {self.name}")
        container = zipfile.ZipFile(self.root, mode='r')
        stream = container.open(self.name, 'r')
        reader = io.TextIOWrapper(stream, **kwargs)
        reader_close = reader.close  # original close

        def close(*args, **kwargs):
            reader_close()
            stream.close()
            container.close()
        reader.close = close   # hijack
        return reader


@dataclass
class TarPath(ArchivedPath):

    def exists(self):
        with tarfile.open(self.root, encoding='utf-8') as root:
            return self.name in root.getnames()

    def open(self, mode='r', **kwargs):
        assert mode in ('r', 'rt'), f'only "r" is supported, given: {mode}'
        log.info(f"Reading from tar file : {self.root} // {self.name}")
        container = tarfile.open(self.root, mode='r', encoding='utf-8')
        stream = container.extractfile(self.name)
        reader = io.TextIOWrapper(stream, **kwargs)
        reader_close = reader.close   # original close

        def close(*args, **kwargs):
            reader_close()
            stream.close()
            container.close()
        reader.close = close   # hijack
        return reader



#!/usr/bin/env python
#
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 1/26/21

from mtdata.main import get_data, lang_pair, DatasetId
from pathlib import Path
from tempfile import TemporaryDirectory


dataset_id = DatasetId.parse


def test_rapid1019():
    # out_dir = Path('tmp-test-tmx1')
    with TemporaryDirectory() as out_dir:
        out_dir = Path(out_dir)
        langs = lang_pair('eng-ces')
        did = dataset_id('Tilde-rapid-2019-ces-eng')
        args = dict(langs=langs, out_dir=out_dir, train_dids=[did])
        get_data(**args)
        parts_dir = out_dir / 'train-parts'
        eng = parts_dir / f'{did}.eng'
        ces = parts_dir / f'{did}.ces'
        eng = eng.read_text().splitlines()
        ces = ces.read_text().splitlines()
        assert len(eng) == len(ces) == 263_287
        assert eng[0].strip() == '(^) Products that are put to sale before 20 May 2016 can still be sold in the EU until 2017.'
        assert ces[0].strip() =='(^) Výrobky, které byly určeny k prodeji před 20. květnem 2016, mohou být v EU stále prodávány do roku 2017.'


def test_eac_ref():
    with TemporaryDirectory() as out_dir:
        out_dir = Path(out_dir)
        langs = lang_pair('nor-slk')
        did = dataset_id('EU-eac_reference-1-nor-slk')

        args = dict(langs=langs, out_dir=out_dir, train_dids=[did])
        get_data(**args)
        parts_dir = out_dir / 'train-parts'
        nor = parts_dir / f'{did}.nor'
        slk = parts_dir / f'{did}.slk'
        nor = nor.read_text().splitlines()
        slk = slk.read_text().splitlines()
        print('nor, slk == ', len(nor), len(slk))
        assert len(nor) == len(slk)



#!/usr/bin/env python
#
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 5/13/20

from mtdata.index import Index, Entry, DatasetId
import itertools

def load_all(index: Index):
    cite = index.ref_db.get_bibtex('ziemski-etal-2016-united')
    url = "https://stuncorpusprod.blob.core.windows.net/corpusfiles/UNv1.0.testsets.tar.gz"
    langs = ['en', 'ar', 'fr', 'es', 'ru', 'zh']
    for split in ['dev', 'test']:
        for l1, l2 in itertools.combinations(langs, 2):
            f1 = f'testsets/{split}set/UNv1.0.{split}set.{l1}'
            f2 = f'testsets/{split}set/UNv1.0.{split}set.{l2}'
            ent = Entry(did=DatasetId(group='UN', name=f'un_{split}', version='1', langs=(l1, l2)),
                 url=url, filename='UNv1.0.testsets.tar.gz', in_ext='txt', in_paths=[f1, f2], cite=cite)
            index.add_entry(ent)



#!/usr/bin/env python
# Corpora from the EU https://ec.europa.eu/jrc/en/language-technologies
# Author: Kenneth Heafield [mtdata (at) kheafield (dot) com] 
# Created: 5/23/21

from mtdata.index import Index, Entry, DatasetId


def load_all(index: Index):
    group = 'europa'
    # === ECDC ===
    # https://ec.europa.eu/jrc/en/language-technologies/ecdc-translation-memory
    cite = index.ref_db.get_bibtex('Steinberger2014')
    langs = 'en bg cs da de el es et fi fr ga hu is it lt lv mt nl no pl pt ro sk sl sv'.split()
    for i, l1 in enumerate(langs):
        for l2 in langs[i+1:]:
            ent = Entry(did=DatasetId(group='EU', name=f'ecdc', version='1', langs=(l1, l2)),
                        url="http://optima.jrc.it/Resources/ECDC-TM/ECDC-TM.zip",
                        in_ext='tmx', cite=cite, in_paths=["ECDC-TM/ECDC.tmx"])
            index.add_entry(ent)

    # === EAC ===
    # https://ec.europa.eu/jrc/en/language-technologies/eac-translation-memory
    # This corpus has two 
    langs = 'bg cs da de el en es et fi fr hu is it lt lv mt nb nl pl pt ro sk sl sv tr'.split()
    for i, l1 in enumerate(langs):
        for l2 in langs[i+1:]:
            ent = Entry(did=DatasetId(group='EU', name=f'eac_forms', version='1', langs=(l1, l2)),
                        url="https://wt-public.emm4u.eu/Resources/EAC-TM/EAC-TM-all.zip",
                        in_ext='tmx', cite=cite, in_paths=["EAC_FORMS.tmx"])
            index.add_entry(ent)
    langs = 'bg cs da de el en es et fi fr hr hu is it lt lv mt nl no pl pt ro sk sl sv tr'.split()
    for i, l1 in enumerate(langs):
        for l2 in langs[i+1:]:
            ent = Entry(did=DatasetId(group='EU', name=f'eac_reference', version='1', langs=(l1, l2)),
                        url="https://wt-public.emm4u.eu/Resources/EAC-TM/EAC-TM-all.zip",
                        in_ext='tmx', cite=cite, in_paths=["EAC_REFRENCE_DATA.tmx"])
            index.add_entry(ent)

    # === DCEP ===
    # https://ec.europa.eu/jrc/en/language-technologies/dcep
    # This was annoying to process so I ended up rehosting it.
    # Don't bother with TR; it doesn't have sentences anyway.
    cite = index.ref_db.get_bibtex('dcep')
    langs = 'BG CS DA DE EL EN ES ET FI FR GA HU IT LT LV MT NL PL PT RO SK SL SV'.split()
    for i, l1 in enumerate(langs):
        for l2 in langs[i+1:]:
            ent = Entry(did=DatasetId(group='EU', name=f'dcep', version='1', langs=(l1, l2)),
                url=f"http://data.statmt.org/DCEP/{l1}-{l2}.tsv.xz", in_ext='tsv', cite=cite)
            index.add_entry(ent)

    # Note: DGT-TM is already in via OPUS.



#!/usr/bin/env python
# All other single corpus
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 5/23/20

from mtdata.index import Index, Entry, DatasetId


def load_all(index: Index):

    # === IITB hin eng http://www.cfilt.iitb.ac.in/iitb_parallel/
    cite = index.ref_db.get_bibtex('Kunchukuttan-etal-iitb')
    l1, l2 = 'hi', 'en'
    for version, prefix in [
        #('1.0', 'http://www.cfilt.iitb.ac.in/iitb_parallel/iitb_corpus_download'),
        ('1.5', 'http://www.cfilt.iitb.ac.in/~moses/iitb_en_hi_parallel/iitb_corpus_download')]:
        # they also have v2, but the link is broken http://www.cfilt.iitb.ac.in/iitb_parallel/
        # version is not explicit, but guessed from file modification time and description
        url = prefix + "/parallel.tgz"
        ent = Entry(did=DatasetId(group='IITB', name=f'hien_train', version=version, langs=(l1, l2)),
                    url=url, filename=f'IITB{version}-hin_eng-parallel.tar.gz',
                    in_ext='txt', cite=cite,
                    in_paths=[f'parallel/IITB.en-hi.{l1}',
                              f'parallel/IITB.en-hi.{l2}'])
        index.add_entry(ent)

        url = prefix + "/dev_test.tgz"
        for split in ['dev', 'test']:
            f1 = f'dev_test/{split}.{l1}'
            f2 = f'dev_test/{split}.{l2}'
            ent = Entry(did=DatasetId(group='IITB', name=f'hien_{split}', version=version, langs=(l1, l2)),
                        url=url, filename=f'IITB{version}-hin_eng-dev_test.tar.gz',
                        in_ext='txt', in_paths=[f1, f2], cite=cite)
            index.add_entry(ent)


    # == Japanese ==
    cite = index.ref_db.get_bibtex('neubig11kftt')
    url = "http://www.phontron.com/kftt/download/kftt-data-1.0.tar.gz"
    l1, l2 = 'en', 'ja'
    for split in ['train', 'test', 'dev', 'tune']:
        f1 = f'kftt-data-1.0/data/orig/kyoto-{split}.{l1}'
        f2 = f'kftt-data-1.0/data/orig/kyoto-{split}.{l2}'
        ent = Entry(did=DatasetId(group='Phontron', name=f'kftt_{split}', version='1', langs=(l1, l2)),
                    url=url, filename="kftt-data-1.0.tar.gz", in_ext='txt', in_paths=[f1, f2], cite=cite)
        index.add_entry(ent)

    url = "http://lotus.kuee.kyoto-u.ac.jp/WAT/my-en-data/wat2020.my-en.zip"
    cite = index.ref_db.get_bibtex('ding2020a')
    for split in ['dev', 'test', 'train']:
        ent = Entry(did=DatasetId(group='WAT', name=f'alt_{split}', version='2020', langs=('my', 'en')),
                    url=url, in_ext='txt', cite=cite, filename='wat2020.my-en.zip',
                    in_paths=[f'wat2020.my-en/alt/{split}.alt.my', f'wat2020.my-en/alt/{split}.alt.en'])
        index.add_entry(ent)

    l1, l2 = 'iu', 'en'
    url = "https://nrc-digital-repository.canada.ca/eng/view/dataset/?id=c7e34fa7-7629-43c2-bd6d-19b32bf64f60"
    cite = index.ref_db.get_bibtex('joanis-etal-2020-nunavut')
    for split in ['dev', 'devtest', 'test', 'train']:
        path_pref = f'Nunavut-Hansard-Inuktitut-English-Parallel-Corpus-3.0/split/{split}'
        if split != 'train':
            path_pref += '-dedup'
        ent = Entry(did=DatasetId(group='NRC_CA', name=f'nunavut_hansard_{split}', version='3', langs=(l1, l2)),
                    url=url, in_ext='txt', cite=cite, filename='NunavutHansard_iuen_v3.tgz',
                    in_paths=[f'{path_pref}.{l1}', f'{path_pref}.{l2}'])
        index.add_entry(ent)

    # https://lindat.mff.cuni.cz/repository/xmlui/handle/11234/1-2122
    url = "https://lindat.mff.cuni.cz/repository/xmlui/bitstream/handle/11234/1-2122/khresmoi-summary-test-set-2.0.zip"
    cite = index.ref_db.get_bibtex('Khresmoi')
    langs = ["cs", "de", "en", "es", "fr", "hu", "pl", "sv"]
    for i, l1 in enumerate(langs):
        for l2 in langs[i+1:]:
            ent = Entry(did=DatasetId(group='Lindat', name=f'khresmoi_summary_test', version='2', langs=(l1, l2)),
                        url=url, filename='khresmoi-summary-test-set-2.0.zip', cite=cite, in_ext='txt',
                        in_paths=[f"khresmoi-summary-test-set-2.0/khresmoi-summary-test.{l1}",
                                  f"khresmoi-summary-test-set-2.0/khresmoi-summary-test.{l2}"])
            index.add_entry(ent)
            ent = Entry(did=DatasetId(group='Lindat', name=f'khresmoi_summary_dev', version='2', langs=(l1, l2)),
                        url=url, filename='khresmoi-summary-test-set-2.0.zip', cite=cite, in_ext='txt',
                        in_paths=[f"khresmoi-summary-test-set-2.0/khresmoi-summary-dev.{l1}",
                                  f"khresmoi-summary-test-set-2.0/khresmoi-summary-dev.{l2}"])
            index.add_entry(ent)

    jesc_url = 'https://nlp.stanford.edu/projects/jesc/data/split.tar.gz'
    jesc_cite = index.ref_db.get_bibtex('pryzant_jesc_2018')
    for split in ['train', 'dev', 'test']:
        ent = Entry(did=DatasetId(group='StanfordNLP', name=f'jesc_{split}', version='1', langs=(l1, l2)),
                    url=jesc_url, filename='jesc-split.tar.gz', in_ext='tsv',
                    in_paths=[f"split/{split}"], cite=jesc_cite)
        index.add_entry(ent)



#!/usr/bin/env python
#
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 5/17/21

from mtdata.index import Index, Entry, DatasetId


def load_all(index: Index):
    group = 'AI4Bharath'
    cite = index.ref_db.get_bibtex('ramesh2021samanantar')
    pairs = ('en-as en-bn en-gu en-hi en-kn en-ml en-mr en-or en-pa en-ta en-te as-bn as-gu as-hi'
             ' as-kn as-ml as-mr as-or as-pa as-ta as-te bn-gu bn-hi bn-kn bn-ml bn-mr bn-or bn-pa'
             ' bn-ta bn-te gu-hi gu-kn gu-ml gu-mr gu-or gu-pa gu-ta gu-te hi-kn hi-ml hi-mr hi-or'
             ' hi-pa hi-ta hi-te kn-ml kn-mr kn-or kn-pa kn-ta kn-te ml-mr ml-or ml-pa ml-ta ml-te'
             ' mr-or mr-pa mr-ta mr-te or-pa or-ta or-te pa-ta pa-te ta-te')
    BASE_v0_2 = 'https://storage.googleapis.com/samanantar-public/V0.2/data/{dirname}/{pair}.zip'
    for pair in pairs.strip().split(' '):
        l1, l2 = pair.split('-')
        dirname = 'en2indic' if l1 == 'en' else 'indic2indic'
        url = BASE_v0_2.format(dirname=dirname, pair=pair)

        ent = Entry(did=DatasetId(group=group, name=f'samananthar', version='0.2', langs=(l1, l2)),
                    url=url, cite=cite, in_paths=[f'{pair}/train.{l1}', f'{pair}/train.{l2}'], in_ext='txt')
        index.add_entry(ent)

    URL = "https://storage.googleapis.com/samanantar-public/benchmarks.zip"
    filename = "samananthar-benchmarks.zip"
    for split in ('dev', 'test'):
        want20_langs = 'bn gu hi ml mr ta te'.split()
        for l2 in want20_langs:
            f1 = f'benchmarks/wat2020-devtest/en-{l2}/{split}.en'
            f2 = f'benchmarks/wat2020-devtest/en-{l2}/{split}.{l2}'
            ent = Entry(did=DatasetId(group=group, name=f'wat_{split}', version='2020', langs=('en', l2)),
                        filename=filename, url=URL, cite=cite, in_paths=[f1, f2], in_ext='txt')
            index.add_entry(ent)

        wat21_langs = 'bn en gu hi kn ml mr or pa ta te'.split()
        for i, l1 in enumerate(wat21_langs):
            for l2 in wat21_langs[i + 1:]:
                f1 = f'benchmarks/wat2021-devtest/{split}.{l1}'
                f2 = f'benchmarks/wat2021-devtest/{split}.{l2}'
                ent = Entry(did=DatasetId(group=group, name=f'wat_{split}', version='2021', langs=(l1, l2)),
                            filename=filename, url=URL, cite=cite, in_paths=[f1, f2], in_ext='txt')
                index.add_entry(ent)

        # PMI langs; en-as
        index.add_entry(Entry(
            did=DatasetId(group=group, name=f'pmi_{split}', version='2021', langs=('en', 'as')),
            filename=filename, url=URL, cite=cite, in_ext='txt',
            in_paths=[f'benchmarks/pmi/en-as/{split}.en', f'benchmarks/pmi/en-as/{split}.as']))



#!/usr/bin/env python
#
#
# Author: Thamme Gowda
# Created: 10/17/21
import collections as coll
import sys
import argparse

from mtdata import __version__, log
from mtdata.index import get_entries, bcp47, is_compatible

exclusions = [
    'jw300-1-',  # we have newer version
    'qed-2.0a-'  # for research purpose only
]


def parse_version(txt):
    # noinspection PyBroadException
    try:
        return float(txt), ''
    except:
        return 1, txt


def filter_english_entries():
    entries = get_entries()
    log.info(f"Found {len(entries):,} sets in my index")
    eng = bcp47('eng')
    entries = [e for e in entries
               if is_compatible(eng, e.did.langs[0]) or is_compatible(eng, e.did.langs[1])]
    log.info(f"Found {len(entries):,} sets have English on either source or target side")
    entries = [e for e in entries if not any(x in str(e.did) for x in exclusions)]
    log.info(f"Found {len(entries):,} after removing exclusions : {exclusions}")
    # versioned entries
    """
    ver_entries = coll.defaultdict(set)
    for e in entries:
        did = e.did
        xxx = did.langs[1].lang if is_compatible(eng, did.langs[0].lang) else did.langs[0].lang
        key = (xxx, did.group, did.name)
        ver_entries[key].add((did.version, e))
    log.info(f"Excluding older versions: {len(ver_entries):,} ")
    lang_entries = coll.defaultdict(set)
    for (xxx, group, name), sets in ver_entries.items():
        # reverse sort (version, entry) items
        sorter_set = sorted(sets, reverse=True, key=lambda x: parse_version(x[0]))
        recent_version = list(sorter_set)[0][1]  # the top 1
        lang_entries[xxx].add(recent_version)
    n_selected_entries = sum(len(v) for v in lang_entries.values())
    """
    lang_entries = coll.defaultdict(set)
    for e in entries:
        did = e.did
        xxx = did.langs[1].lang if is_compatible(eng, did.langs[0].lang) else did.langs[0].lang
        lang_entries[xxx].add(e)
    n_selected_entries = sum(len(v) for v in lang_entries.values())
    log.info(f"Total languages={len(lang_entries):,} || total datasets selected={n_selected_entries:,}")
    # sort based on descending order of number of entries
    lang_entries = list(sorted(lang_entries.items(), key=lambda x: len(x[1]), reverse=True))
    sep = '\t'
    print('Lang', '#sets', 'Trains', 'Devs', 'Tests', sep=sep)
    for i, (xxx, sets) in enumerate(lang_entries, start=1):
        devs = []
        tests = []
        trains = []
        for s in sets:
            name = s.did.name
            did = str(s.did)
            if 'test' in name:
                tests.append(did)
            elif 'dev' in name:
                devs.append(did)
            else:
                trains.append(did)
        trains, devs, tests = [x and ', '.join(sorted(x)) or '' for x in [trains, devs, tests]]
        print(xxx, len(sets), trains, devs, tests, sep=sep)


def main(**kwargs):
    filter_english_entries()


def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('-vv', '--verbose', action='store_true', help='verbose mode')
    parser.add_argument('-v', '--version', action='version', version=f'%(prog)s {__version__}')

    parser.add_argument("-i", "--inp", type=argparse.FileType('r', encoding='utf-8'), default=sys.stdin,
                        help="Input file. Default: STDIN")
    parser.add_argument("-o", "--out", type=argparse.FileType('w', encoding='utf-8'), default=sys.stdin,
                        help="Input file. Default: STDOUT")
    args = parser.parse_args()
    if args.inp is sys.stdin or args.out is sys.stdout:
        assert sys.getdefaultencoding().lower() in ('utf-8', 'utf8'), \
            f'Please set PYTHONIOENCODING=utf-8; Current encoding: {sys.getdefaultencoding()}'
    return args


if __name__ == '__main__':
    main()



#!/usr/bin/env python
#
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 4/4/20
import argparse
from pathlib import Path
from collections import defaultdict

import mtdata
from mtdata import log, __version__, cache_dir as CACHE_DIR, cached_index_file
from mtdata.entry import DatasetId, LangPair
from mtdata.utils import IO
from mtdata.iso.bcp47 import bcp47


def list_data(langs, names, not_names=None, full=False):
    from mtdata.index import get_entries
    entries = get_entries(langs, names, not_names, fuzzy_match=True)
    log.info(f"Found {len(entries)}")
    for i, ent in enumerate(entries):
        print(ent.format(delim='\t'))
        if full:
            print(ent.cite or "CITATION_NOT_LISTED", end='\n\n')
    print(f"Total {len(entries)} entries")


def get_data(langs, out_dir, train_dids=None, test_dids=None, dev_did=None, merge_train=False, compress=False,
             drop_dupes=False, drop_tests=False, **kwargs):
    if kwargs:
        log.warning(f"Args are ignored: {kwargs}")
    from mtdata.data import Dataset
    assert train_dids or test_dids, 'Required --train or --test or both'
    dataset = Dataset.prepare(
        langs, train_dids=train_dids, test_dids=test_dids, out_dir=out_dir,
        dev_did=dev_did, cache_dir=CACHE_DIR, merge_train=merge_train, compress=compress,
        drop_dupes=drop_dupes, drop_tests=drop_tests)
    cli_sig = f'-l {"-".join(str(l) for l in langs)}'
    if train_dids:
        cli_sig += f' -tr {" ".join(str(d) for d in train_dids)}'
    if test_dids:
        cli_sig += f' -ts {" ".join(str(d) for d in test_dids)}'
    if dev_did:
        cli_sig += f' -dv {dev_did}'
    for flag, val in [('--merge', merge_train), ('--compress', compress), ('-dd', drop_dupes), ('-dt', drop_tests)]:
        if val:
            cli_sig += ' ' + flag
    sig = f'mtdata get {cli_sig} -o <out-dir>\nmtdata version {mtdata.__version__}\n'
    log.info(f'Dataset is ready at {dataset.dir}')
    log.info(f'mtdata args for reproducing this dataset:\n {sig}')
    with IO.writer(out_dir / 'mtdata.signature.txt', append=True) as w:
        w.write(sig)


def generate_report(langs, names, not_names=None, format='plain'):
    from mtdata.index import get_entries
    entries = get_entries(langs, names, not_names)
    lang_stats = defaultdict(int)
    name_stats = defaultdict(int)
    group_stats = defaultdict(int)
    for ent in entries:
        lang_stats[ent.lang_str] += 1
        name_stats[ent.did.name] += 1
        group_stats[ent.did.group] += 1

    print("Languages:")
    for key, val in lang_stats.items():
        print(f'{key}\t{val:,}')

    print("\nNames:")
    for key, val in name_stats.items():
        print(f'{key}\t{val:,}')

    print("\nGroups:")
    for key, val in group_stats.items():
        print(f'{key}\t{val:,}')


def list_experiments(args):
    raise Exception("Not implemented yet")


class MyFormatter(argparse.ArgumentDefaultsHelpFormatter):

    def _split_lines(self, text, width: int):
        if text.startswith("R|"):
            return text[2:].splitlines()
        return super()._split_lines(text, width)


def lang_pair(string) -> LangPair:
    parts = string.split('-')
    if len(parts) != 2:
        msg = f'expected value of form "xxx-yyz" eg "deu-eng"; given {string}'
        raise argparse.ArgumentTypeError(msg)
    std_codes = (bcp47(parts[0]), bcp47(parts[1]))
    std_form = '-'.join(str(lang) for lang in std_codes)
    if std_form != string:
        log.info(f"Suggestion: Use codes {std_form} instead of {string}."
                 f" Let's make a little space for all languages of our planet 😢.")
    return std_codes


def add_boolean_arg(parser: argparse.ArgumentParser, name, dest=None, default=False, help=''):
    group = parser.add_mutually_exclusive_group()
    dest = dest or name
    group.add_argument(f'--{name}', action='store_true', dest=dest, default=default, help=help)
    group.add_argument(f'--no-{name}', action='store_false', dest=dest, default=not default,
                       help='Do not ' + help)


def parse_args():
    p = argparse.ArgumentParser(formatter_class=MyFormatter, epilog=f'Loaded from {__file__} (v{__version__})')
    p.add_argument('-vv', '--verbose', action='store_true', help='verbose mode')
    p.add_argument('-v', '--version', action='version', version=f'%(prog)s {__version__}')
    p.add_argument('-ri', '--reindex', action='store_true',
                   help=f"Invalidate index of entries and recreate it. This deletes"
                        f" {cached_index_file} only and not the downloaded files. "
                        f"Use this if you're using in developer mode and modifying mtdata index.")

    sub_ps = p.add_subparsers(required=True, dest='task',
                              help='''R|
"list" - List the available entries 
"get" - Downloads the entry files and prepares them for experiment
"list-exp" - List the (well) known papers and datasets used in their experiments 
"get-exp" - Get the datasets used in the specified experiment from "list-exp" 
''')

    list_p = sub_ps.add_parser('list', formatter_class=MyFormatter)
    list_p.add_argument('-l', '--langs', metavar='L1-L2', type=lang_pair,
                        help='Language pairs; e.g.: deu-eng')
    list_p.add_argument('-n', '--names', metavar='NAME', nargs='*',
                        help='Name of dataset set; eg europarl_v9.')
    list_p.add_argument('-nn', '--not-names', metavar='NAME', nargs='*', help='Exclude these names')
    list_p.add_argument('-f', '--full', action='store_true', help='Show Full Citation')
    list_p.add_argument('-o', '--out', type=Path, help='This arg is ignored. '
                                                       'Only used in "get" subcommand.')

    get_p = sub_ps.add_parser('get', formatter_class=MyFormatter)
    get_p.add_argument('-l', '--langs', metavar='L1-L2', type=lang_pair,
                       help='Language pairs; e.g.: deu-eng', required=True)
    get_p.add_argument('-tr', '--train', metavar='ID', dest='train_dids', nargs='*', type=DatasetId.parse,
                       help='''R|Names of datasets separated by space, to be used for *training*.
    e.g. -tr Statmt-news_commentary-16-deu-eng europarl_v9 .
     To concatenate all these into a single train file, set --merge flag.''')
    get_p.add_argument('-ts', '--test', metavar='ID', dest='test_dids', nargs='*', type=DatasetId.parse,
                       help='''R|Names of datasets separated by space, to be used for *testing*. 
    e.g. "-ts Statmt-newstest_deen-2019-deu-eng Statmt-newstest_deen-2020-deu-eng ".
    You may also use shell expansion if your shell supports it.
    e.g. "-ts Statmt-newstest_deen-20{19,20}-deu-eng" ''')
    get_p.add_argument('-dv', '--dev', metavar='ID', dest='dev_did', type=DatasetId.parse, required=False,
                       help='''R|Dataset to be used for development (aka validation). 
    e.g. "-dev Statmt-newstest_deen-2017-deu-eng"''')
    add_boolean_arg(get_p, 'merge', dest='merge_train', default=False, help='Merge train into a single file')
    get_p.add_argument(f'--compress', action='store_true', default=False, help="Keep the files compressed")
    get_p.add_argument('-dd', f'--dedupe', '--drop-dupes', dest='drop_dupes', action='store_true', default=False,
                       help="Remove duplicate (src, tgt) pairs in training (if any); valid when --merge. "
                            "Not recommended for large datasets. ")
    get_p.add_argument('-dt', f'--drop-tests', dest='drop_tests', action='store_true', default=False,
                       help="Remove dev/test sentences from training sets (if any); valid when --merge")
    get_p.add_argument('-o', '--out', dest='out_dir', type=Path, required=True, help='Output directory name')

    report_p = sub_ps.add_parser('report', formatter_class=MyFormatter)
    report_p.add_argument('-l', '--langs', metavar='L1-L2', type=lang_pair,
                          help='Language pairs; e.g.: deu-eng')
    report_p.add_argument('-n', '--names', metavar='NAME', nargs='*',
                          help='Name of dataset set; eg europarl_v9.')
    report_p.add_argument('-nn', '--not-names', metavar='NAME', nargs='*', help='Exclude these names')

    args = p.parse_args()
    if args.verbose:
        log.getLogger().setLevel(level=log.DEBUG)
        mtdata.debug_mode = True
    return args


def main():
    args = parse_args()
    if args.reindex and cached_index_file.exists():
        bak_file = cached_index_file.with_suffix(".bak")
        log.info(f"Invalidate index: {cached_index_file} -> {bak_file}")
        cached_index_file.rename(bak_file)

    if args.task == 'list':
        list_data(args.langs, args.names, not_names=args.not_names, full=args.full)
    elif args.task == 'get':
        get_data(**vars(args))
    elif args.task == 'list_exp':
        list_experiments(args)
    elif args.task == 'report':
        generate_report(args.langs, names=args.names, not_names=args.not_names)
    else:
        raise Exception(f'{args.task} not implemented')


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# This module along with accompanying bcp47.json, tries to parse BCP47 like language IDs
# This is not 100% compatible with BCP47
# We diverge on :
#   using three letter code for all languages
#      => language: 3 letter (lowercase), script: 4 letter (title case), and country: 2 letter (uppercase)
#   no support for too many variations: limited to (lll, Ssss, CC) i.e (lang, script, country)
#
# Author: Thamme Gowda [tg (at) isi.edu]
# Created: 10/3/21

import json
from collections import namedtuple
from pathlib import Path
from typing import Optional, Union
from functools import lru_cache
from mtdata.iso import iso3_code


def load_json(path: Path):
    assert path.exists()
    with open(path, encoding='utf-8') as fp:
        return json.load(fp)


class BCP47Tag(namedtuple('BCP47Tag', ('lang', 'script', 'region', 'tag'))):
    __slots__ = ()
    joiner = '_'  # per BCP47, we must use '-' hyphen not underscore, but we use '-' to separate languages e.g. eng-deu

    def __new__(cls, lang, script: Optional[str] = None, region: Optional[str] = None, tag: Optional[str] = None):
        tag = tag or ''.join([lang,
                              f'{cls.joiner}{script}' if script else '',
                              f'{cls.joiner}{region}' if region else ''])
        obj = super(BCP47Tag, cls).__new__(cls, lang, script, region, tag)
        # obj.tag = tag
        return obj

    def __str__(self):
        return self.tag

    def __repr__(self):
        return f'BCP47({self.tag})'

    def __lt__(self, other):
        return self.tag > other.tag

    def is_compatible(self, lang2: Union[str, 'BCP47Tag']):

        if isinstance(lang2, str):
            lang2 = bcp47(lang2)
        # exact same tag => true
        if self.tag == lang2.tag:
            return True
        # languages or scripts are different
        if self.lang != lang2.lang or self.script != lang2.script:
            return False
        # one of them is general, other is a variant of region
        # e.g. eng vs eng_US
        if not self.region or not lang2.region:
            return True
        return False

    @classmethod
    def are_compatible(cls, tag1, tag2):
        tag1 = tag1 if isinstance(tag1, cls) else bcp47(tag1)
        return tag1.is_compatible(tag2)


class BCP47Parser:

    def __init__(self, data):
        self.data = data
        assert all(key in data for key in ['languages', 'scripts', 'countries']), 'malformed bcp4j data'
        self.scripts = {code: name for code, name in data['scripts']}
        self.countries = {code: name for code, name in data['countries']}
        self.languages = {code3: (code2, name) for code3, code2, name in data['languages']}
        for key in self.languages:  # validation
            assert key == iso3_code(key, fail_error=True)
        self.default_scripts = {}  # these needs suppression; eng-Latn is just eng, as Latn is default
        for lang_code, script_code, lang_name in data['default_scripts']:
            code3 = iso3_code(lang_code, fail_error=True)
            assert script_code in self.scripts
            self.default_scripts[code3] = script_code

    @lru_cache(maxsize=10000)
    def parse(self, tag) -> BCP47Tag:
        """
        Parameters
        ----------
        tag : tag to be parsed

        Returns
        -------
            BCP47Tag
        """
        code_orig = tag
        tag = tag.replace('_', '-').strip()
        assert tag
        parts = tag.split('-')
        assert 1 <= len(parts) <= 3, f'BCP47 code longer than 3 parts not supported yet; given {code_orig}'
        lang, script, region = None, None, None
        # part 1: it has to be language
        lang = iso3_code(parts[0], default=None)
        if not lang or lang not in self.languages:
            raise ValueError(f'Unable to recognize {code_orig}; Unknown language')
        # assert lang in self.languages, f'Language "{lang}" is invalid; input: {code_orig}'

        parts = parts[1:]
        if parts:  # part 2 can be either script or region code
            if parts[0].title() in self.scripts:
                script = parts[0].title()
            elif parts[0].upper() in self.countries:
                region = parts[0].upper()
            elif parts[0] == 'XX':      # placeholder for a country
                pass
            else:
                raise ValueError(f'Unable to parse {code_orig}')
            parts = parts[1:]
        if parts:  # part 3, if it exists, must be a region
            assert script
            assert not region
            if parts[0].upper() in self.countries:
                region = parts[0].upper()
            else:
                raise ValueError(f"Cant find {code_orig}; Unknown region")
            parts = parts[1:]
        assert not parts  # all parts are consumed
        if script and lang in self.default_scripts and self.default_scripts[lang] == script:
            script = None  # suppress script
        return BCP47Tag(lang=lang, script=script, region=region)

    def try_parse(self, tag, default=None):
        """
        Tries to parse a language tag; upon failure, returns the default value
        Parameters
        ----------
        tag : tag to be parsed
        default : default value to return upon failure

        Returns
        -------
        """
        try:
            return self(tag)
        except ValueError:
            return default

    def __call__(self, tag) -> BCP47Tag:
        """
        Parameters
        ----------
        tag : tag to be parsed
        Returns
        -------
           BCP47Tag
        """
        if isinstance(tag, BCP47Tag):
            return tag
        return self.parse(tag)


data_file = Path(__file__).parent / "bcp47.json"
bcp47_data = load_json(data_file)
bcp47 = BCP47Parser(data=bcp47_data)

if __name__ == '__main__':
    import argparse

    p = argparse.ArgumentParser(prog='python -m mtdata.iso.bcp47', description="BCP47 lookup tool")
    p.add_argument("langs", nargs='+', help="Language code or name that needs to be looked up. "
                                            "When no language code is given, all languages are listed.")
    args = vars(p.parse_args())
    if args.get('langs'):
        print("INPUT\tSTD\tLANG\tSCRIPT\tREGION")
        for inp in args['langs']:
            tag = bcp47(inp)
            print(f'{inp}\t{tag}\t{tag.lang}\t{tag.script}\t{tag.region}')



#!/usr/bin/env python
#
#
# Author: Thamme Gowda
# Created: 10/13/21

from mtdata.entry import Entry, DatasetId


def test_dataset_id():

    obj1 = DatasetId(group='uscisi', name='dataset1', version='1', langs=('en', 'kn'))
    obj2 = DatasetId(group='uscisi', name='dataset1', version='1', langs=('en', 'kn'))
    obj3 = DatasetId(group='uscisi', name='dataset1', version='2', langs=('en', 'kn'))
    obj4 = DatasetId(group='uscisi', name='dataset1', version='1', langs=('en', 'fr'))
    obj5 = DatasetId(group='opus', name='dataset1', version='1', langs=('en', 'kn'))

    assert obj2 == obj1
    assert obj3 != obj1
    assert obj4 != obj1
    assert obj5 != obj1

    mem = set()
    assert obj1 not in mem   # obj1 should be hashable to work
    mem.add(obj1)
    assert obj1 in mem      # obj1 should be hashable to work
    assert obj2 in mem      # this should work! obj2 is a dupe of obj1
    assert obj3 not in mem

    mem = dict()
    mem[obj1] = 10
    assert obj1 in mem
    assert obj2 in mem
    assert mem[obj2] == 10
    assert obj3 not in mem



#!/usr/bin/env python
#
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 4/5/20

from typing import Union
from pathlib import Path
from xml.etree import ElementTree as ET
import argparse
from mtdata import log
from mtdata.utils import IO
import time
from mtdata.iso.bcp47 import bcp47
from mtdata.main import lang_pair
from html import unescape
import datetime

DEF_PROGRESS = 10  # seconds


def parse_tmx(data, log_every=DEF_PROGRESS):
    context = ET.iterparse(data, events=['end'])
    tus = (el for event, el in context if el.tag == 'tu')
    count = 0
    st = t = time.time()
    for tu in tus:
        lang_seg = {}
        for tuv in tu.findall('tuv'):
            lang = [v for k, v in tuv.attrib.items() if k.endswith('lang')]
            seg = tuv.findtext('seg')
            if lang and seg:
                lang = bcp47(lang[0])
                seg = unescape(seg.strip()).replace('\n', ' ').replace('\t', ' ')
                if lang in lang_seg:
                    log.warning(f"Language {lang} appears twice in same translation unit.")
                lang_seg[lang] = seg
        yield lang_seg
        count += 1
        if log_every and (time.time() - t) > log_every:
            elapsed = datetime.timedelta(seconds=round(time.time() - st))
            log.info(f"{elapsed} :: Parsed: {count:,}")
            t = time.time()
        tu.clear()


def read_tmx(path: Union[Path, str], langs=None):
    """
    reads a TMX file as records
    :param path: path to .tmx file
    :param langs: (lang1, lang2) codes eg (de, en); when it is None the code tries to auto detect
    :return: stream of (text1, text2)
    """
    passes = 0
    fails = 0
    with IO.reader(path) as data:
        recs = parse_tmx(data)
        for lang_seg in recs:
            if langs is None:
                log.warning("langs not set; this could result in language mismatch")
                if len(lang_seg) == 2:
                    langs = tuple(lang_seg.keys())
                else:
                    raise Exception(f"Language autodetect for TMX only supports 2 languages,"
                                    f" but provided with {lang_seg.keys()} in TMX {path}")
            if langs[0] in lang_seg and langs[1] in lang_seg:
                yield lang_seg[langs[0]], lang_seg[langs[1]]
                passes += 1
            else:
                fails += 1
    if passes == 0:
        if fails == 0:
            raise Exception(f"Empty TMX {path}")
        raise Exception(f"Nothing for {langs[0]}-{langs[1]} in TMX {path}")
    if fails != 0:
        log.warning(f"Skipped {fails} entries due to language mismatch in TMX {path}")
    log.info(f"Extracted {passes} pairs from TMX {path}")


def main(inp, out, langs):
    recs = read_tmx(inp, langs=langs)
    with IO.writer(out) as out:
        count = 0
        for rec in recs:
            rec = [l.replace('\t', ' ') for l in rec]
            out.write('\t'.join(rec) + '\n')
            count += 1
        log.warning(f"Wrote {count} lines to {out}")


if __name__ == '__main__':
    p = argparse.ArgumentParser(description='A tool to convert TMX to TSV',
                                formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    p.add_argument('-i', '--inp', type=Path, required=True, help='Input file path')
    p.add_argument('-o', '--out', type=Path, default=Path('/dev/stdout'),
                   help='Output file path')
    p.add_argument('-l', '--langs', type=lang_pair, default=None,
                   help='Languages from TMX. example: eng-fra or en-fr')

    args = vars(p.parse_args())
    main(**args)



#!/usr/bin/env python
#
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 4/8/20
import collections
import pickle
from pathlib import Path
from typing import List, Dict, Union

import portalocker
from pybtex.database import parse_file as parse_bib_file

from mtdata import log, cached_index_file, __version__
from mtdata.entry import Entry, Paper, DatasetId, LangPair
from mtdata.iso.bcp47 import bcp47, BCP47Tag

REFS_FILE = Path(__file__).parent / 'refs.bib'


class Index:

    obj = None   # singleton object

    def __init__(self):
        self.entries: Dict[DatasetId, Entry] = {}  # unique dids
        self.papers = {}                            # unique
        self.ref_db = ReferenceDb()
        self.version = __version__

    @classmethod
    def get_instance(cls):
        if not cls.obj:
            if not cached_index_file.exists():
                log.info("Creating a fresh index object")
                cached_index_file.parent.mkdir(exist_ok=True)
                lock_file = cached_index_file.with_suffix("._lock")
                with portalocker.Lock(lock_file, 'w', timeout=60) as fh:
                    # got lock, check cache is not created by parallel processes while we waited
                    if not cached_index_file.exists():
                        obj = Index()
                        log.info("Indexing all datasets...")
                        obj.load_all()
                        log.info(f"Caching my index file at {cached_index_file}")
                        with open(cached_index_file, 'wb') as out:
                            pickle.dump(obj, out)

            assert cached_index_file.exists()
            log.info(f"Loading index from cache {cached_index_file}")
            with open(cached_index_file, 'rb') as inp:
                obj = pickle.load(inp)

            assert isinstance(obj, cls), f'{cached_index_file} isnt valid. please move or remove it'
            cls.obj = obj
        return cls.obj

    def load_all(self):
        from mtdata.index import (
            statmt, paracrawl, tilde, joshua_indian, unitednations, wikimatrix, other, neulab_tedtalks,
            elrc_share, ai4bharat, eu, linguatools)
        from mtdata.index.opus import opus_index, jw300, opus100
        subsets = [
            ('Statmt.org', statmt.load),
            ('Paracrawl', paracrawl.load),
            ('Tilde', tilde.load),
            ('JoshuaIndianCoprus', joshua_indian.load_all),
            ('UnitedNations', unitednations.load_all),
            ('OPUS', opus_index.load_all),
            ('OPUS_JW300', jw300.load_all),
            ('OPUS100', opus100.load_all),
            ('WikiMatrix', wikimatrix.load_all),
            ('Other', other.load_all),
            ('Neulab_TEDTalksv1', neulab_tedtalks.load_all),
            ('ELRC-SHARE', elrc_share.load_all),
            ('AI4Bharat', ai4bharat.load_all),
            ('EU', eu.load_all),
            ('LinguaTools', linguatools.load_all)
        ]
        for name, loader in subsets:
            loader(self)

        counts = collections.defaultdict(int)
        for e in self.entries.values():
            counts[e.did.group] += 1
        items = list(sorted(counts.items(), key=lambda x: x[1], reverse=True))
        items += [('Total', len(self))]
        counts = '\n'.join([f'| {n} | {c:,}|' for n, c in items])
        log.info(f"Index status:\n{counts}")

    @property
    def n_entries(self) -> int:
        return len(self.entries)

    def add_entry(self, entry: Entry):
        assert isinstance(entry, Entry)
        key = entry.did
        assert key not in self.entries, f'{key} is a duplicate'
        self.entries[key] = entry

    def add_paper(self, paper: Paper):
        assert isinstance(paper, Paper)
        assert paper.name not in self.papers, f'{paper.name} is a duplicate'
        self.papers[paper.name] = paper

    def get_entries(self):
        return self.entries.values()

    def get_papers(self):
        return self.papers.values()

    def __contains__(self, item):
        assert isinstance(item, DatasetId)
        return item in self.entries

    def __getitem__(self, item):
        assert isinstance(item, DatasetId)
        return self.entries[item]

    def contains_paper(self, name):
        return name in self.papers

    def get_entry(self, name, langs):
        assert isinstance(name, str)
        assert isinstance(langs, tuple)
        langs = tuple(lang if isinstance(lang, BCP47Tag) else bcp47(lang) for lang in langs)
        key = (name, langs)
        rev_key = (name, tuple(reversed(langs)))
        if key not in self.entries and rev_key in self.entries:
            key = rev_key
        return self.entries[key]

    def get_paper(self, name):
        return self.papers[name]

    def __len__(self):
        return len(self.entries)


class ReferenceDb:

    _instance = None  # singleton instance

    def __new__(cls, file=REFS_FILE):
        if cls._instance is None:
            cls._instance = super(ReferenceDb, cls).__new__(cls)
            assert file.exists(), f'{file} does not exist'
            cls._instance.db = parse_bib_file(file, bib_format='bibtex')
            log.debug(f"loaded {len(cls._instance)} references from {file}")
        return cls._instance

    def __getitem__(self, item):
        return self.db.entries[item]

    def __contains__(self, item):
        return item in self.db.entries

    def __len__(self):
        return len(self.db.entries)

    def get_bibtex(self, key: str) -> str:
        return self[key].to_string(bib_format='bibtex')

    def keys(self):
        return self.db.entries.keys()


def is_compatible(lang1: Union[str, BCP47Tag], lang2: Union[str, BCP47Tag]):
    lang1 = lang1 if isinstance(lang1, BCP47Tag) else bcp47(lang1)
    lang2 = lang2 if isinstance(lang2, BCP47Tag) else bcp47(lang2)
    return lang1.is_compatible(lang2)


def bitext_lang_match(pair1, pair2, fuzzy_match=False) -> bool:
    x1, y1 = sorted(pair1)
    x2, y2 = sorted(pair2)
    if fuzzy_match:
        return is_compatible(x1, x2) and is_compatible(y1, y2)
    else:
        return x1 == x2 and y1 == y1


def get_entries(langs=None, names=None, not_names=None, fuzzy_match=False) -> List[Entry]:
    """
    :param langs: language pairs  to select eg ('en', 'de')
    :param names:  names to select
    :param not_names:  names to exclude
    :param fuzzy_match
    :return: list of dataset entries that match the criteria
    """
    # TODO: our index has grown too big; improve search with fuzzy matches
    select = list(INDEX.get_entries())
    if names:
        names = set(n.lower() for n in names)
        select = [e for e in select if e.did.name in names]
    if langs:
        assert len(langs) == 2
        langs = sorted(langs)
        select = [e for e in select if bitext_lang_match(langs, e.did.langs, fuzzy_match=fuzzy_match)]
    if not_names:
        if not isinstance(not_names, set):
            not_names = set(not_names)
        select = [e for e in select if e.did.name not in not_names]
    return select


INDEX: Index = Index.get_instance()



#!/usr/bin/env python
#
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 4/4/20
import errno
from mtdata import main, log

if __name__ == '__main__':
    try:
        main.main()
    except BrokenPipeError as e:
        # this happens when piped to '| head' which aborts pipe after limit. And that is okay.
        pass



#!/usr/bin/env python
#
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 5/16/21

data = '''
ns,nso
sz,szl
tz,zgh
zz,zza
cb,ckb
cx,bsb
sh,hbs
in,ind
iw,heb
jp,jpn
jap,jpn
daf,dnj
'''
# these are manual mappings from https://wp-info.org/tools/languagecodes.php

CUSTOM_TO_3 = {}
for line in data.splitlines():
    line = line.strip()
    if not line:
        continue
    xx, iso = line.split(',')
    CUSTOM_TO_3[xx] = iso



#!/usr/bin/env python
#
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 4/5/20
from pathlib import Path
from xml.etree import ElementTree as ET
import argparse
from mtdata import log
from mtdata.utils import IO
from typing import Iterator
import re
from html import unescape


def clean(text):
    """
    unescape
    remove unnecessary white spaces
    replace \\n and \\t with regular space
    :param text:
    :return:
    """
    return ' '.join(unescape(text).split())


def read_sgm_xml(data: Path) -> Iterator[str]:
    """Extract sgm using XML parse
    This reader crashes if there is any error in XML e.g. an & is not escaped ;
      see newstest2019-frde-ref.de.sgm for example!
    """
    with IO.reader(data) as data:
        context = ET.iterparse(data, events=['end'])
        segs = (el for event, el in context if el.tag == 'seg')
        count = 0
        for seg in segs:
            yield clean(seg.text)
            seg.clear()
            count += 1
        log.info(f"read {count} segments from {data}")

def read_sgm_regex(data: Path) -> Iterator[str]:
    """
    Extract sgm using regex.
    assumes each sgm is in its own line of form <seg id="xx"> yy</sgm>
    and line breaks are used between
    :param data:
    :return:
    """
    patt = re.compile(r'<seg id="(.*)">(.*)</seg>')
    count = 0
    with IO.reader(data) as data:
        for line in data:
            line = line.strip()
            match = patt.search(line)
            if match:
                yield clean(match.group(2))
                count += 1
    log.info(f"read {count} segments from {data}")

# alias
read_sgm = read_sgm_regex
#read_sgm = read_sgm_xml


def read_wmt21_xml(data):
    """
    This is a new XML format (instead of SGM) introduced in WMT2021
    :param data:
    :return:
    """
    tree = ET.parse(data)
    # 1. Assumes exactly one translation
    # 2. buffering in memory.  this can be improved with stream parsing
    def xpath_all(tree, xpath):
        return (clean(seg.text) for seg in tree.findall(xpath))

    srcs = list(xpath_all(tree.getroot(), xpath=".//src//seg"))
    tgts = list(xpath_all(tree.getroot(), xpath=".//ref//seg"))
    assert len(srcs) == len(tgts), f'{data} has unequal number of segs: {len(srcs)} == {len(tgts)}?'
    yield from zip(srcs, tgts)
    log.info(f"Read {len(srcs)} segs from {data}")

def main(inp, out, wmt21xml=False):
    parser = read_wmt21_xml if wmt21xml else read_sgm
    stream = parser(inp)
    with IO.writer(out) as out:
        count = 0
        for rec in stream:
            if isinstance(rec, str):
                rec = (rec,)
            line = '\t'.join(rec) + '\n'
            out.write(line)
            count += 1
        log.info(f"Wrote {count} lines to {out}")


if __name__ == '__main__':
    import sys
    p = argparse.ArgumentParser(description='A tool to convert TMX to TSV',
                                formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    p.add_argument('-i', '--inp', type=Path, required=True, help='Input file path')
    p.add_argument('-o', '--out', type=Path, default=sys.stdout, help='Output file path')
    p.add_argument('--wmt21xml', action='store_true', help='Input is wmt21 XML format')
    args = vars(p.parse_args())
    main(**args)



#!/usr/bin/env python
#
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 5/13/20
from mtdata.index import Index, Entry, DatasetId


def load_all(index: Index):

    cite = index.ref_db.get_bibtex(key='post-etal-2012-constructing')
    url = 'https://github.com/joshua-decoder/indian-parallel-corpora/archive/a2cd1a99.tar.gz'
    l2 = 'en'
    langs = ['ml', 'hi', 'ur', 'bn', 'te', 'ta']
    for l1 in langs:
        for split in ['training', 'dev', 'test', 'devtest', 'dict']:
            if l1 == 'hi' and split == 'dict':
                continue   # hindi dont have dict
            f1 = f'*/{l1}-{l2}/{split}.{l1}-{l2}.{l1}'
            f2 = f'*/{l1}-{l2}/{split}.{l1}-{l2}.{l2}'
            if split not in ('training', 'dict'):
                f2 += '.0'
            ent = Entry(did=DatasetId(group='JoshuaDec', name=f'indian_{split}', version='1', langs=(l1, l2)),
                  url=url, filename='joshua-indian-parallel-corpora.tar.gz',
                  in_paths=[f1, f2], in_ext='txt', cite=cite)
            index.add_entry(ent)
